/*
 *  Gtk_CoordBox.cc
 * 
 *  Fractal Generator
 * 
 *  Copyright (c) 1998 Brian Martin
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include "Gtk_CoordBox.h"
#include "fractal.h"

Gtk_CoordBox::Gtk_CoordBox(Gtk_CoordBox_State * _state): 
    Gtk_Frame(_state->label),
    state(_state)
{
    box = new Gtk_VBox(False, 0);
    button_box = new Gtk_HBox(False, 0);
    x = state->x.entry = new Gtk_HEntryScale(&state->x);
    y = state->y.entry = new Gtk_HEntryScale(&state->y);
    z = state->z.entry = new Gtk_HEntryScale(&state->z);
    constrain = new Gtk_CheckButton("constrain");
    
    revert = new Gtk_Button("revert");

    constrained = false;

    button_box->pack_start(constrain, TRUE, TRUE, 0);
    button_box->pack_start(revert, TRUE, TRUE, 0);

    box->pack_start(x, TRUE, TRUE, 2);
    box->pack_start(y, TRUE, TRUE, 2);
    box->pack_start(z, TRUE, TRUE, 2);
    box->pack_start(button_box, TRUE, TRUE, 2);

    x->show();
    y->show();
    z->show();
    constrain->show();   
    revert->show();

    button_box->show();

    add(box);
    box->show();

    show();

    connect_to_signal(x->getAdjustment()->value_changed, changed);
    connect_to_signal(y->getAdjustment()->value_changed, changed);
    connect_to_signal(z->getAdjustment()->value_changed, changed);
    connect_to_method(constrain->toggled,this,&Gtk_CoordBox::toggle_constrain);
    connect_to_method(revert->clicked, this, &Gtk_CoordBox::revert_values);
}

void Gtk_CoordBox::revert_values() {
    state->apply();
}

void Gtk_CoordBox::toggle_constrain() {

  //    printf("toggle_constrain\n");

    if(constrained) {
	constrained = false;
	
	Gtk_Adjustment * adj;
	adj = new Gtk_Adjustment(x->getVal(),
				 x->getMinval(),
				 x->getMaxval(),
				 x->getRes(),
				 0, 0);
	connect_to_signal(adj->value_changed, changed);
 	y->setAdjustment(adj);
	gtk_object_unref(GTK_OBJECT(adj->gtkobject));
//adj->unref();
	adj = new Gtk_Adjustment(x->getVal(),
				 x->getMinval(),
				 x->getMaxval(),
				 x->getRes(),
				 0, 0);
	connect_to_signal(adj->value_changed, changed);
 	z->setAdjustment(adj);
	gtk_object_unref(GTK_OBJECT(adj->gtkobject));
	x->update();
	y->update();
	z->update();
//	adj->unref();
    } else {
	constrained = true;

	float new_val = (x->getVal() + y->getVal() + z->getVal())/3;
	float new_upper = (x->getMaxval() + y->getMaxval() + z->getMaxval())/3;
	float new_lower = (x->getMinval() + y->getMinval() + z->getMinval())/3;
	float new_res = (x->getRes() + y->getRes() + z->getRes())/3;
	Gtk_Adjustment *new_adj = new Gtk_Adjustment(new_val, new_lower,
						     new_upper, new_res, 0, 0);
	x->setAdjustment(new_adj);
	y->setAdjustment(new_adj);
	z->setAdjustment(new_adj);
	gtk_object_unref(GTK_OBJECT(new_adj->gtkobject));
	connect_to_signal(new_adj->value_changed, changed);
//	new_adj->unref();
    }
}

// void Gtk_CoordBox::reset_bounds(double x_minval,
// 				double x_maxval,
// 				double x_currval,
// 				double y_minval,
// 				double y_maxval,
// 				double y_currval,
// 				double z_minval,
// 				double z_maxval,
// 				double z_currval) {

//     if(constrained) {
// 	Gtk_Adjustment * new_adj = new Gtk_Adjustment(x_currval,
// 						      x_minval,
// 						      x_maxval,
// 						      0.1, 0 , 0);
// 	x->setAdjustment(new_adj);
// 	y->setAdjustment(new_adj);
// 	z->setAdjustment(new_adj);
// 	connect_to_signal(new_adj->value_changed, changed);
//     } else {
// 	x->reset(x_minval,
// 		 x_maxval,
// 		 x_currval);
// 	connect_to_signal(x->getAdjustment()->value_changed, changed);
// 	y->reset(y_minval,
// 		 y_maxval,
// 		 y_currval);
// 	connect_to_signal(y->getAdjustment()->value_changed, changed);
// 	z->reset(z_minval,
// 		 z_maxval,
// 		 z_currval);
// 	connect_to_signal(z->getAdjustment()->value_changed, changed);
//     }
// }

double Gtk_CoordBox::getVal(int which) {

    switch(which) {

    case X:
	return x->getVal();
	break;

    case Y:
	return y->getVal();
	break;

    case Z:
	return z->getVal();
	break;
    }
}

