#include "eq_table.h"
#include "expr_parser.h"
#include <stdlib.h>
#include <string.h>
#include <math.h>

eq_table::eq_table(eq_table *copy) {

    index_of_last_column = copy->index_of_last_column;
    q_index = copy->q_index;
    table = new Entry*[index_of_last_column+1];
    for(int i = 0 ; i <= index_of_last_column ; i++) {
	table[i] = new Entry(copy->table[i]);
    }
}

eq_table::Entry::Entry(Entry * copy) {
    
    for(int i = 0 ; i < 4 ; i++) {
	vect[i] = copy->vect[i];
	props[i] = copy->props[i];
    }
    index = copy->index;
    expr = copy->expr;
    skip = copy->skip;
}

eq_table::eq_table(expr_ref expr, int _max_iter, double _max_val) {
  
    max_iter = _max_iter;
    max_val = _max_val;

    table_list = 0;

    c_index = -1;

    add_column(expr);
    
    for(Entry * ptr = table_list ; ptr ; ptr = ptr->next) {
	switch (ptr->expr->token_code) {

	case 'c':
	    c_index = ptr->index;
	case 'q':
	    if(ptr->expr->token_code == 'q')
		q_index = ptr->index;
	case 'i':
	case 'j':
	case 'k':
	case TOK_LIT_QUAT:
	    for(int context = 0 ; context < 4 ; context++ ) {
		ptr->props[context].rule = 1;
		ptr->props[context].dep1 = -1; // no deps
		ptr->props[context].dep2 = -1;
	    }
	    break;
		
	case 'e':
	case TOK_LIT_PI:
	case TOK_LIT_SCALAR:
	    for(int context = 0 ; context < 4 ; context++ ) {
		ptr->props[context].rule = 2;
		ptr->props[context].dep1 = -1; // no deps
		ptr->props[context].dep2 = -1;
	    }
	    break;

	case '+':
	    for(int context = 0 ; context < 4 ; context++ ) {
		ptr->props[context].rule = 3;
		ptr->props[context].dep1 = add_column(ptr->expr->lchld);
		ptr->props[context].dep2 = add_column(ptr->expr->rchld);
	    }
	    break;

	case '-':
	    for(int context = 0 ; context < 4 ; context++ ) {
		ptr->props[context].rule = 4;
		ptr->props[context].dep1 = add_column(ptr->expr->lchld);
		ptr->props[context].dep2 = add_column(ptr->expr->rchld);
	    }
	    break;
				
	case '*':
	    for(int context = 0 ; context < 4 ; context++ ) {
		ptr->props[context].rule = 5+context;
		ptr->props[context].dep1 = add_column(ptr->expr->lchld);
		ptr->props[context].dep2 = add_column(ptr->expr->rchld);
	    }
	    break;

	case '^':
	    if(ptr->expr->lchld->scalar) { // scalar base
		for(int context = 0 ; context < 4 ; context++) {
		    ptr->props[context].rule = 12+context;
		    ptr->props[context].dep1 = add_column(ptr->expr->lchld);
		    ptr->props[context].dep2 = add_column(ptr->expr->rchld);
		}
	    } else {		// quaternion base
		if(atoi(ptr->expr->rchld->data) & 1) { // odd exponent
		    for(int context = 0 ; context < 4 ; context++ ) {
			ptr->props[context].rule = 5+context;
			ptr->props[context].dep1 = add_column(ptr->expr->lchld);
		    }

		    char buf[30];

		    expr_ref new_child = ptr->expr->copy();
		    sprintf(buf, "%d", atoi(new_child->rchld->data)-1);
		    new_child->rchld->data = strdup(buf);
		
		    for(int context = 0 ; context < 4 ; context++ )   
			ptr->props[context].dep2 = add_column(new_child);

		} else {		// even exponent
		    ptr->props[0].rule = 10;
		    ptr->props[1].rule = ptr->props[2].rule =
			ptr->props[3].rule = 11;
					    
		    char buf[30];
		    expr_ref new_child = ptr->expr->copy();
		    int power = atoi(new_child->rchld->data)/2;
		    if(power == 1) {
			ptr->props[0].dep1 = ptr->props[1].dep1 = 
			    ptr->props[2].dep1 = ptr->props[3].dep1 = 
			    add_column(new_child->lchld);
			delete new_child->rchld;
			delete new_child;
		    } else {
			sprintf(buf, "%d", atoi(new_child->rchld->data)/2);
			new_child->rchld->data = strdup(buf);
			ptr->props[0].dep1 = ptr->props[1].dep1 = 
			    ptr->props[2].dep1 = ptr->props[3].dep1 = 
			    add_column(new_child);
		    }
		}
	    }
	    break;
	}
    }

    Entry * old_table = table_list;

    for(old_table=table_list;old_table->next;old_table=old_table->next);

    index_of_last_column = old_table->index;
    old_table = table_list;

    table = new Entry *[index_of_last_column+1];
    for(int i = 0 ; i <= index_of_last_column ; i++) {
	table[i] = old_table;
	old_table = old_table->next;
    }

				// now we need to look through table and insert
				// values for all the literals
    for(int i = 0 ; i <= index_of_last_column ; i++) {
	if(table[i]->expr->token_code == TOK_LIT_QUAT ||
	   table[i]->expr->token_code == 'i' ||
	   table[i]->expr->token_code == 'j' ||
	   table[i]->expr->token_code == 'k') {
	    table[i]->skip = true;
	    table[i]->vect[0] = table[i]->expr->val->s;
	    table[i]->vect[1] = table[i]->expr->val->a;
	    table[i]->vect[2] = table[i]->expr->val->b;
	    table[i]->vect[3] = table[i]->expr->val->c;
	    printf("added literal quat to table - %.2f %.2f %.2f %.2f\n",
		   table[i]->expr->val->s, table[i]->expr->val->a, 
		   table[i]->expr->val->b, table[i]->expr->val->c);
	} else if(table[i]->expr->token_code == TOK_LIT_SCALAR ||
		  table[i]->expr->token_code == TOK_LIT_PI ||
		  table[i]->expr->token_code == 'e') {
	    table[i]->skip = true;
	    table[i]->vect[0] = table[i]->expr->scalar_val;
	    table[i]->vect[1] = table[i]->expr->scalar_val;
	    table[i]->vect[2] = table[i]->expr->scalar_val;
	    table[i]->vect[3] = table[i]->expr->scalar_val;
	} else {
	    table[i]->skip = false;
	}
    }    
    table[q_index]->skip = true;
    if(c_index != -1)
	table[c_index]->skip = true;    
} 

int eq_table::add_column(expr_ref expr) {

    Entry * ptr;
				// look for expr in table
    for(ptr = table_list ; ptr ; ptr = ptr->next) 
	if(ptr->expr->eql(expr))
	    return ptr->index;

				// expr not found in table 
    Entry * new_entry = new Entry();

    new_entry->expr = expr;

    if(table_list) {
	for(ptr = table_list ; ptr->next ; ptr = ptr->next);
    	new_entry->index = ptr->index + 1;
	ptr->next = new_entry;
    } else {
	new_entry->index = 0;
	table_list = new_entry;
    }

    return new_entry->index;
}

double eq_table::iter(double s, double a, double b, double c, int & iters) {
    
    table[q_index]->vect[0] = s;
    table[q_index]->vect[1] = a;
    table[q_index]->vect[2] = b;
    table[q_index]->vect[3] = c;

    if(c_index != -1) {
	table[c_index]->vect[0] = s;
	table[c_index]->vect[1] = a;
	table[c_index]->vect[2] = b;
	table[c_index]->vect[3] = c;
    }

//     printf("p is %.2f %.2f %.2f %.2f\n", s, a, b, c);

    int iter = 0;

    do {
	eval();
// 	memcpy(table[0]->vect,
// 	       table[q_index]->vect,
// 	       sizeof(double)*4);
	table[q_index]->vect[0] = table[0]->vect[0];
	table[q_index]->vect[1] = table[0]->vect[1];
	table[q_index]->vect[2] = table[0]->vect[2];
	table[q_index]->vect[3] = table[0]->vect[3];
//  	printf("p is %.2f %.2f %.2f %.2f\n", table[0]->vect[0],
//  	       table[0]->vect[1],table[0]->vect[2],table[0]->vect[3]);
    } while(++iter < max_iter && mag() < max_val);
    
    iters = iter;

    return(mag());
}

double eq_table::mag() {	// returns mag^2
    return(table[0]->vect[0]*table[0]->vect[0]+
	   table[0]->vect[1]*table[0]->vect[1]+
	   table[0]->vect[2]*table[0]->vect[2]+
	   table[0]->vect[3]*table[0]->vect[3]);
}

void eq_table::eval() {

//     table[q_index]->vect[0] = q->s;
//     table[q_index]->vect[1] = q->a;
//     table[q_index]->vect[2] = q->b;
//     table[q_index]->vect[3] = q->c;
    
    Entry * curr;

    for(int i = index_of_last_column ; i >= 0 ; i--) {
	if(table[i]->skip)
	    continue;
	curr = table[i];

	for(int context = 0 ; context < 4 ; context++) {
	    int larg = curr->props[context].dep1;
	    int rarg = curr->props[context].dep2;
	    double log_of_base;
	    
	    switch (curr->props[context].rule) {

	    case 3:		// c_e(x+y)
		curr->vect[context] = table[larg]->vect[context] +
		    table[rarg]->vect[context];
		break;

	    case 4:		// c_e(x-y)
		curr->vect[context] = table[larg]->vect[context] -
		    table[rarg]->vect[context];
		break;

	    case 5:		// c_s(x*y)
		curr->vect[context] = 
		    table[larg]->vect[0]*table[rarg]->vect[0] -
		    table[larg]->vect[1]*table[rarg]->vect[1] -
		    table[larg]->vect[2]*table[rarg]->vect[2] -
		    table[larg]->vect[3]*table[rarg]->vect[3];
		break;

	    case 6:		// c_a(x*y)
 		curr->vect[context] = 
		    table[larg]->vect[1]*table[rarg]->vect[0] +
		    table[larg]->vect[0]*table[rarg]->vect[1] +
		    table[larg]->vect[2]*table[rarg]->vect[3] -
		    table[larg]->vect[3]*table[rarg]->vect[2];
		break;

	    case 7:		// c_b(x*y)
 		curr->vect[context] = 
		    table[larg]->vect[2]*table[rarg]->vect[0] +
		    table[larg]->vect[0]*table[rarg]->vect[2] +
		    table[larg]->vect[3]*table[rarg]->vect[1] -
		    table[larg]->vect[1]*table[rarg]->vect[3];
		break;

	    case 8:		// c_c(x*y)
 		curr->vect[context] = 
		    table[larg]->vect[3]*table[rarg]->vect[0] +
		    table[larg]->vect[0]*table[rarg]->vect[3] +
		    table[larg]->vect[1]*table[rarg]->vect[2] -
		    table[larg]->vect[2]*table[rarg]->vect[1];
		break;

	    case 10:		// c_s(x^y), y even
		curr->vect[context] = 
		    table[larg]->vect[0]*table[larg]->vect[0] -
		    table[larg]->vect[1]*table[larg]->vect[1] -
		    table[larg]->vect[2]*table[larg]->vect[2] -
		    table[larg]->vect[3]*table[larg]->vect[3];
		break;

	    case 11:		// c_e(x^y), y even, e is a or b or c
		curr->vect[context] = 
		    2*table[larg]->vect[context]*table[larg]->vect[0];
		break;

	    case 12:		// c_s(s^q)
		log_of_base = log(table[larg]->vect[0]);
		curr->vect[context] =
		    cos(log_of_base*table[rarg]->vect[1])*
		    cos(log_of_base*table[rarg]->vect[2])*
		    cos(log_of_base*table[rarg]->vect[3]) - 
		    sin(log_of_base*table[rarg]->vect[1])*
		    sin(log_of_base*table[rarg]->vect[2])*
		    sin(log_of_base*table[rarg]->vect[3]);
		break;

	    case 13:		// c_a(s^q)
		curr->vect[context] =
		    cos(log_of_base*table[rarg]->vect[1])*
		    sin(log_of_base*table[rarg]->vect[2])*
		    sin(log_of_base*table[rarg]->vect[3]) +
		    sin(log_of_base*table[rarg]->vect[1])*
		    cos(log_of_base*table[rarg]->vect[2])*
		    cos(log_of_base*table[rarg]->vect[3]);
		break;

	    case 14:		// c_b(s^q)
		curr->vect[context] =
		    cos(log_of_base*table[rarg]->vect[1])*
		    sin(log_of_base*table[rarg]->vect[2])*
		    cos(log_of_base*table[rarg]->vect[3]) - 
		    sin(log_of_base*table[rarg]->vect[1])*
		    cos(log_of_base*table[rarg]->vect[2])*
		    sin(log_of_base*table[rarg]->vect[3]);
		break;

	    case 15:		// c_c(s^q)
		curr->vect[context] =
		    cos(log_of_base*table[rarg]->vect[1])*
		    cos(log_of_base*table[rarg]->vect[2])*
		    sin(log_of_base*table[rarg]->vect[3]) + 
		    sin(log_of_base*table[rarg]->vect[1])*
		    sin(log_of_base*table[rarg]->vect[2])*
		    cos(log_of_base*table[rarg]->vect[3]);
		break;
	    }
	}
    }
//     q->set(table[0]->vect[0],
// 	   table[0]->vect[1],
// 	   table[0]->vect[2],
// 	   table[0]->vect[3]);
    
//     return new quat(table[0]->vect[0],
// 		    table[0]->vect[1],
// 		    table[0]->vect[2],
// 		    table[0]->vect[3]);
}

void eq_table::set_init_q_val(double * v) {
    init_q_val[0] = v[0];
    init_q_val[1] = v[1];
    init_q_val[2] = v[2];
    init_q_val[3] = v[3];
    q_is_c_initially = false;
}

void eq_table::set_q_to_c_initially() {
    q_is_c_initially = true;
}
