%{
#include "defs.h"
#define MATCH(s) return(match(s))
#ifndef YY_USE_PROTOS
#define YY_USE_PROTOS
#endif

%}


DIGIT [0-9]
FLOAT ({DIGIT}+("."{DIGIT}+)?)|("."{DIGIT}+)


%%

[*^/+()qijkce-]                   { MATCH(*yytext); }
pi                          { MATCH(TOK_LIT_PI); }
{FLOAT}                     { MATCH(TOK_LIT_SCALAR); }
\[" "*-?{FLOAT}" "*-?{FLOAT}" "*-?{FLOAT}" "*-?{FLOAT}" "*\]    { MATCH(TOK_LIT_QUAT); }
[\n\t ]+                   {  }
.                          { char buf[1024] ; sprintf(buf, "unrecognized token '%s'\n", yytext) ; yyerror(buf); }

%%


int match(int token_code) {

    expr_ref ans = new Expr_Tree(yytext, token_code);

    if(token_code == TOK_LIT_QUAT) {
	double a, b, c, d;
	sscanf(yytext, "[ %lf %lf %lf %lf ]", &a, &b, &c, &d);
	ans->setVal(new quat(a, b, c, d));
	ans->set_constant();
    } else if(token_code == TOK_LIT_SCALAR) {
	double s;
	sscanf(yytext, " %lf ", &s);
        ans->setVal(s);
	ans->set_scalar();
	ans->set_constant();
    } else if (token_code == 'e') {
	ans->setVal(2.71828182846);
	ans->set_scalar();
	ans->set_constant();
    } else if (token_code == TOK_LIT_PI) {
	ans->setVal(3.1415926535897);
	ans->set_scalar();
	ans->set_constant();
    } else if (token_code == 'i') {
	ans->setVal(new quat(0, 1, 0, 0));
	ans->set_constant();
    } else if (token_code == 'j') {
	ans->setVal(new quat(0, 0, 1, 0));
	ans->set_constant();
    } else if (token_code == 'k') {
	ans->setVal(new quat(0, 0, 0, 1));
	ans->set_constant();
    }

    yylval = ans;

    return (token_code);
}

int yywrap() {}





