/*
 *  expr_tree.cc
 * 
 *  Fractal Generator
 * 
 *  Copyright (c) 1998 Brian Martin
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "defs.h"
#include <math.h>

char *Expr_Tree::make_string() {
    
    char buf[200];
    buf[0] = 0;
    char *ptr = 0;

    if(lchld) {
	ptr = lchld->make_string();
	strcat(buf, ptr);
	delete ptr;
    }

    strcat(buf, data);

    if(rchld) {
	ptr = rchld->make_string();
	strcat(buf, ptr);
	delete ptr;
    }
    return strdup(buf);
}

void Expr_Tree::dump() {
    
    if(lchld || rchld)
	printf(" ( ");
    
    if(lchld)
 	lchld->dump();

    if(scalar) 
	printf("%.2f", scalar_val);
    else if(val)
	val->dump();
    else 
	printf("%s", data);

    if(rchld)
 	rchld->dump();

    if(lchld || rchld)
	printf(" ) ");
}

/*
  This routine attempts to fold constants as much as possible
 */
expr_ref Expr_Tree::set_children(expr_ref _lchld, expr_ref _rchld) {

    rchld = _rchld;
    lchld = _lchld;

    printf("set_childrenfuck\n");
    
    printf("expr is ");
    dump();
    printf("\n");

    if(rchld->scalar && lchld->scalar) {
	printf("hooptafuck\n");
	scalar = true;
	constant = true;
	switch (token_code) {

	case '*':
	    scalar_val = rchld->scalar_val * lchld->scalar_val;
	    break;

	case '/':
	    scalar_val = lchld->scalar_val / rchld->scalar_val;
	    break;

	case '+':
	    scalar_val = rchld->scalar_val + lchld->scalar_val;
	    break;

	case '-':
	    scalar_val = lchld->scalar_val - rchld->scalar_val;
	    break;

	case '^':
	    scalar_val = pow(lchld->scalar_val, rchld->scalar_val);
	    break;
	    
	default:
	    fprintf(stderr, "Unrecognized operator %d\n", token_code);
	    exit(-1);
	    break;
	}
	token_code = TOK_LIT_SCALAR;
	delete rchld;
	delete lchld;
	lchld = rchld = NULL;
	printf("folded constants - %.2f\n", scalar_val);
    } else if (rchld->constant && lchld->constant) {
	constant = true;
	if(lchld->scalar) {

	    double log_base;
	    double base_s;

	    val = new quat(rchld->val);
	    
	    switch (token_code) {
	    case '*':
		val->s *= lchld->scalar_val;
		val->a *= lchld->scalar_val;
		val->b *= lchld->scalar_val;
		val->c *= lchld->scalar_val;
		break;

	    case '+':
		val->s += lchld->scalar_val;
		val->a += lchld->scalar_val;
		val->b += lchld->scalar_val;
		val->c += lchld->scalar_val;
		break;

	    case '-':
		val->s -= lchld->scalar_val;
		val->a -= lchld->scalar_val;
		val->b -= lchld->scalar_val;
		val->c -= lchld->scalar_val;
		break;

	    case '^':		// more difficult
		log_base = log(lchld->scalar_val);
		base_s = pow(lchld->scalar_val, rchld->val->s);
		val->s = base_s * (cos(rchld->val->a*log_base)*
				   cos(rchld->val->b*log_base)*
				   cos(rchld->val->c*log_base) -
				   sin(rchld->val->a*log_base)*
				   sin(rchld->val->b*log_base)*
				   sin(rchld->val->c*log_base));
		val->a = base_s * (cos(rchld->val->a*log_base)*
				   sin(rchld->val->b*log_base)*
				   sin(rchld->val->c*log_base) +
				   sin(rchld->val->a*log_base)*
				   cos(rchld->val->b*log_base)*
				   cos(rchld->val->c*log_base));
		val->b = base_s * (cos(rchld->val->a*log_base)*
				   sin(rchld->val->b*log_base)*
				   cos(rchld->val->c*log_base) -
				   sin(rchld->val->a*log_base)*
				   cos(rchld->val->b*log_base)*
				   sin(rchld->val->c*log_base));
		val->c = base_s * (cos(rchld->val->a*log_base)*
				   cos(rchld->val->b*log_base)*
				   sin(rchld->val->c*log_base) +
				   sin(rchld->val->a*log_base)*
				   sin(rchld->val->b*log_base)*
				   cos(rchld->val->c*log_base));
		break;
		
	    default:
		fprintf(stderr, "Unrecognized operator %d\n", token_code);
		exit(-1);
		break;

	    }
	    token_code = TOK_LIT_QUAT;
	} else if (rchld->scalar) {
	    val = new quat(lchld->val);
	    
	    switch (token_code) {
	    case '*':
		val->s *= rchld->scalar_val;
		val->a *= rchld->scalar_val;
		val->b *= rchld->scalar_val;
		val->c *= rchld->scalar_val;
		break;

	    case '/':
		val->s /= rchld->scalar_val;
		val->a /= rchld->scalar_val;
		val->b /= rchld->scalar_val;
		val->c /= rchld->scalar_val;
		break;

	    case '+':
		val->s += rchld->scalar_val;
		val->a += rchld->scalar_val;
		val->b += rchld->scalar_val;
		val->c += rchld->scalar_val;
		break;

	    case '-':
		val->s -= rchld->scalar_val;
		val->a -= rchld->scalar_val;
		val->b -= rchld->scalar_val;
		val->c -= rchld->scalar_val;
		break;

	    case '^':		// more difficult
				// umm...
		fprintf(stderr, "not implemented yet ;<\n");
		exit(-1);
		break;

	    default:
		fprintf(stderr, "Unrecognized operator %d\n", token_code);
		exit(-1);
		break;
	    }
	    token_code = TOK_LIT_QUAT;
	} else {		// no scalars involved
	    val = new quat;
	    switch (token_code) {
	    case '*':
		val->s = 
		    rchld->val->s*lchld->val->s -
		    rchld->val->a*lchld->val->a -
		    rchld->val->b*lchld->val->b -
		    rchld->val->c*lchld->val->c;
		val->a = 
		    lchld->val->a*rchld->val->s +
		    lchld->val->s*rchld->val->a +
		    lchld->val->b*rchld->val->c -
		    lchld->val->c*rchld->val->b;
		val->b = 
		    lchld->val->b*rchld->val->s +
		    lchld->val->s*rchld->val->b +
		    lchld->val->c*rchld->val->a -
		    lchld->val->a*rchld->val->c;
		val->c = 
		    lchld->val->c*rchld->val->s +
		    lchld->val->s*rchld->val->c +
		    lchld->val->a*rchld->val->b -
		    lchld->val->b*rchld->val->a;
		break;

	    case '+':
		val->s = rchld->val->s+lchld->val->s;
		val->a = rchld->val->a+lchld->val->a;
		val->b = rchld->val->b+lchld->val->b;
		val->c = rchld->val->c+lchld->val->c;
		break;

	    case '-':
		val->s = rchld->val->s-lchld->val->s;
		val->a = rchld->val->a-lchld->val->a;
		val->b = rchld->val->b-lchld->val->b;
		val->c = rchld->val->c-lchld->val->c;
		break;
		
	    default:
		fprintf(stderr, "Unrecognized operator %d\n", token_code);
		exit(-1);
		break;
	    }
	    token_code = TOK_LIT_QUAT;
	}
	delete rchld;
	delete lchld;
	lchld = rchld = NULL;
	printf("folded constants - %.2f %.2f %.2f %.2f\n", val->s, val->a, val->b, val->c);
    }

    printf("after expr is ");
    dump();
    printf("\n");

    return(this);
}

Expr_Tree::Expr_Tree(char * _data, int _token_code) {

    data = strdup(_data);
    token_code = _token_code;
    
    lchld = rchld = 0;
    val = 0;
    scalar = false;
    scalar_val = 0;
    constant = false;
}

Expr_Tree::~Expr_Tree() {

    if(val)
	delete val;
    if(data)
	delete data;

}

bool Expr_Tree::eql(Expr_Tree * comp) {
    
    return(this && comp && 
	   ((constant && 
	     ((scalar && comp->scalar_val == scalar_val) ||
	      (val && comp->val && val->eql(comp->val)))) ||
	    (!val && !comp->val && strcmp(data, comp->data)==0)) &&
	    ((lchld && lchld->eql(comp->lchld)) || !lchld) &&
	    ((rchld && rchld->eql(comp->rchld)) || !rchld));
}

// expr_ref Expr_Tree::load(FILE * file) {
//     int count;
//     int token_code;
//     char buf[200];
//     expr_ref curr;
//     expr_ref old;
//     expr_ref ret_val;

//     fscanf(file, " %d ", &count);

//     fscanf(file, " %s %d ", buf, &token_code);
//     old = ret_val = new Expr_Tree(buf, token_code);

//     for(int i = 1 ; i < count ; i++) {
// 	fscanf(file, " %s %d ", buf, &token_code);
// 	curr = new Expr_Tree(buf, token_code);
// 	curr->rear = old;
// 	old->next = curr;
//     }
// }

// void Expr_Tree::save(FILE * file) {
//     int count = 0;
//     for(expr_ref curr = this ; curr ; curr = curr->next) count++;
//     fprintf(file, "%d\n", count);
//     for(expr_ref curr = this ; curr ; curr = curr->next) {
// 	fprintf(file, "%s\n %d\n", curr->data, curr->token_code);
// 	if(curr->val)
// 	    curr->val->save(file);
//     }
// }

Expr_Tree * Expr_Tree::copy() {
    
    Expr_Tree * copy = new Expr_Tree(data,token_code);
    if(lchld)
      copy->lchld = lchld->copy();
    if(rchld)
      copy->rchld = rchld->copy();
    copy->val = val;    
    return copy;
}
