/*
 *  fract_params.cc
 * 
 *  Fractal Generator
 * 
 *  Copyright (c) 1998 Brian Martin
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <math.h>
#include "fract_params.h"
#include "fractal.h"
#include "defs.h"
#include "statelist.h"

extern StateList fractals;
extern struct yy_buffer_state * yy_scan_string(const char *);
extern void normalizeNormal(fcoord *, void *);

				// updates fractal params from gui
void Fract_Params::apply() {

    bool need_recompute = false;
    bool need_refresh = false;
    bool need_subdivide = false;

    if(strcmp(gen_func->get_text().c_str(), state->expr_string)!=0 ||
       strcmp(init_func->get_text().c_str(), state->init_expr_string)!=0) {
	state->set_expr(gen_func->get_text().c_str(), 
			init_func->get_text().c_str());
	need_recompute = true;
	printf("expr_set\n");
    }

    if(state->w_fix.changed()) {
	state->w_fix.grab();
	need_recompute = true;
    }
    
    if(state->zoom.changed()) {
	need_subdivide = true;
// 	state->zoom.apply();
    }

    if(state->pan.changed()) {
	need_subdivide = true;
// 	state->pan.apply();
    }

//     if(state->size.changed()) {	
// 	need_recompute_sizechange = true;
// //	state->size.grab();
// 	printf("size changed\n");
//     } else 
// 	printf("size not changed\n");

    if(state->bounding_res.changed()) {
	need_refresh = true;
	state->bounding_res.grab();
    }

    printf("queueing requests\n");

    if(need_recompute)
	state->bounding_box_visible = false;

//     if(need_recompute_sizechange)
// 	state->fract->request(FRACT_RECOMPUTE_SIZECHANGE_REQ);

    printf("sizechange\n");
    
    if(state->fract && need_recompute)
	state->fract->request(FRACT_RECOMPUTE_REQ);
    
    printf("recompute\n");

    if(state->fract && need_subdivide) {	// should make this a request
#if 0
	if(state->maxiter.changed()) {
	    state->fract->table->max_iter = (int)(state->maxiter.gui_value());
	    state->maxiter.grab();
	}
#endif
	state->fract->tree->subdivide(state->tree_apply_weight.gui_value(),
	    state->fract->tree->span.x/2 * (1 - state->zoom.x.gui_value())
	    + state->fract->tree->min.x + state->pan.x.gui_value(),
	    
	    state->fract->tree->span.y/2 * (1 - state->zoom.y.gui_value())
	    + state->fract->tree->min.y + state->pan.y.gui_value(),
	    
	    state->fract->tree->span.z/2 * (1 - state->zoom.z.gui_value())
	    + state->fract->tree->min.z + state->pan.z.gui_value(),
	    
	    state->fract->tree->span.x/2 * (1 + state->zoom.x.gui_value())
	    + state->fract->tree->min.x + state->pan.x.gui_value(),
	    
	    state->fract->tree->span.y/2 * (1 + state->zoom.y.gui_value())
	    + state->fract->tree->min.y + state->pan.y.gui_value(),
	    
	    state->fract->tree->span.z/2 * (1 + state->zoom.z.gui_value())
	    + state->fract->tree->min.z + state->pan.z.gui_value());
    
	update_stats();

 	state->fract->params->points->foreach(normalizeNormal, 
 					      (void*)(state->fract));
    }
				// remove this need_subdivide when it is
				// made into a request
    if(state->fract && (need_subdivide || need_recompute || need_refresh))
	state->fract->request(FRACT_REFRESH_REQ);    

    printf("requests queued\n");
}

Fract_Params::Fract_Params(Fract_Params_State * _state):
    state(_state)
{
    notebook = new Gtk_Notebook;
    gen_func = new Gtk_Entry;

    constant_s = new Gtk_Entry;
    constant_i = new Gtk_Entry;
    constant_j = new Gtk_Entry;
    constant_k = new Gtk_Entry;
    
    top_box = new Gtk_VBox(False, 0);
    misc_box = new Gtk_VBox(False, 0);
    zoom_box = new Gtk_VBox(False, 0);
    Gtk_VBox * func_box = new Gtk_VBox(False, 0);

    func_label = new Gtk_Label("f(q): ");

    state_options = new Gtk_OptionMenu;
    state_menu = new Gtk_Menu;
    Gtk_MenuItem * item;

    for(GList * ptr = fractals.get_list() ; ptr ; ptr = ptr->next) 
	if(ptr->data) {
	    item = new Gtk_MenuItem(((fractal_State*)ptr->data)->name);
	    state_menu->append(*item);
	    item->show();
	}

    connect_to_method(fractals.append, this, &Fract_Params::append_fractal);

    state_options->set_menu(*state_menu);
    state_options->show();

    state->zoom.box = new Gtk_CoordBox(&state->zoom);
    state->pan.box = new Gtk_CoordBox(&state->pan);
    state->w_fix.entry = new Gtk_HEntryScale(&state->w_fix);

    button_box = new Gtk_HBox(False, 0);
    dismiss_button = new Gtk_Button("dismiss");
    apply_button = new Gtk_Button("apply");

    bounding_frame = new Gtk_Frame("Bounding Box");
    show_bounding_box = new Gtk_CheckButton("show");
    bounding_box = new Gtk_VBox(False, 0);
    state->bounding_res.entry = new Gtk_HEntryScale(&state->bounding_res);
    
    bounding_box->pack_start(*state->bounding_res.entry, TRUE, TRUE, 4);
    bounding_box->pack_start(*show_bounding_box, TRUE, TRUE, 4);
    show_bounding_box->show();
    state->bounding_res.entry->show();
    bounding_frame->add(bounding_box);
    bounding_box->show();

    misc_box->pack_start(*state->w_fix.entry, TRUE, TRUE, 4);
    misc_box->pack_start(*bounding_frame, TRUE, TRUE, 4);
    notebook->append_page(*misc_box, *(new Gtk_Label("WTF?")));
    zoom_box->pack_start(*state->zoom.box, TRUE, TRUE, 4);
    zoom_box->pack_start(*state->pan.box, TRUE, TRUE, 4);
    notebook->append_page(*zoom_box, *(new Gtk_Label("Zoom")));

    bounding_frame->show();
    misc_box->show();
    zoom_box->show();

    state->w_fix.entry->show();
    state->zoom.box->show();
    state->pan.box->show();

    button_box->pack_start(*apply_button, TRUE, TRUE, 0);
    button_box->pack_start(*dismiss_button, TRUE, TRUE, 0);

    GTK_WIDGET_SET_FLAGS(apply_button->gtkobj(), GTK_CAN_DEFAULT);
    GTK_WIDGET_SET_FLAGS(dismiss_button->gtkobj(), GTK_CAN_DEFAULT);

    gen_func->set_text(state->expr_string);

    Gtk_Label * fract_label = new Gtk_Label("Fractal:");
    Gtk_Table * table = new Gtk_Table(1, 4, 1);
    table->attach_defaults(*fract_label, 0, 1, 0, 1);
    table->attach_defaults(*state_options, 1, 4, 0, 1);
    table->show();    
    
    Gtk_HBox * h_func_box = new Gtk_HBox(False, 0);
    h_func_box->pack_start(*func_label, TRUE, TRUE, 2);
    h_func_box->pack_start(*gen_func, 0, 0, 2);
    func_box->pack_start(*h_func_box, TRUE, TRUE, 0);

    Gtk_HBox * init_func_box = new Gtk_HBox(False, 0);
    Gtk_Label * init_func_label = new Gtk_Label("q_0: ");
    init_func_box->pack_start(*init_func_label, TRUE, TRUE, 4);
    init_func = new Gtk_Entry;
    init_func->set_text(state->init_expr_string);
    init_func_box->pack_start(*init_func, 0, 0, 4);
    func_box->pack_start(*init_func_box, TRUE, TRUE, 0);

    init_func_label->show();
    init_func->show();
    init_func_box->show();

    state->maxiter.entry = new Gtk_HEntryScale(&state->maxiter);
    func_box->pack_start(*state->maxiter.entry, TRUE, TRUE, 0);
    state->maxiter.entry->show();

    Gtk_Table * const_table = new Gtk_Table(5, 6, 1);

    Gtk_Label * s_label = new Gtk_Label("s:");
    const_table->attach_defaults(*s_label, 0, 1, 0, 1);
    const_table->attach_defaults(*constant_s, 1, 6, 0, 1);
    
    Gtk_Label * i_label = new Gtk_Label("i:");
    const_table->attach_defaults(*i_label, 0, 1, 1, 2);
    const_table->attach_defaults(*constant_i, 1, 6, 1, 2);

    Gtk_Label * j_label = new Gtk_Label("j:");
    const_table->attach_defaults(*j_label, 0, 1, 2, 3);
    const_table->attach_defaults(*constant_j, 1, 6, 2,3 );
    
    Gtk_Label * k_label = new Gtk_Label("k:");
    const_table->attach_defaults(*k_label, 0, 1, 3, 4);
    const_table->attach_defaults(*constant_k, 1, 6, 3, 4);

    Gtk_CheckButton * use_this_c = new Gtk_CheckButton("Use this as c");
    const_table->attach_defaults(*use_this_c, 0, 3, 4, 5);

    Gtk_Button * pick = new Gtk_Button("pick constant");
    const_table->attach_defaults(*pick, 3, 6, 4, 5);

    pick->show();

    Gtk_Frame * constant_frame = new Gtk_Frame("Constant");
    constant_frame->add(const_table);

    func_box->pack_start(*constant_frame, TRUE, TRUE, 0);

    s_label->show();
    i_label->show();
    j_label->show();
    k_label->show();

    use_this_c->show();

    constant_s->set_text(state->const_s);
    constant_i->set_text(state->const_a);
    constant_j->set_text(state->const_b);
    constant_k->set_text(state->const_c);

    constant_s->show();
    constant_i->show();
    constant_j->show();
    constant_k->show();
    
    const_table->show();
    constant_frame->show();

    notebook->append_page(*func_box, *(new Gtk_Label("Function")));

				// tree stuffs

    state->tree_center.box = new Gtk_CoordBox(&state->tree_center);
    state->tree_span.box = new Gtk_CoordBox(&state->tree_span);
    state->init_tree_depth.entry = new 
	Gtk_HEntryScale(&state->init_tree_depth);
    state->tree_bisect_iterations.entry = 
	new Gtk_HEntryScale(&state->tree_bisect_iterations);
    state->tree_apply_weight.entry = 
	new Gtk_HEntryScale(&state->tree_apply_weight);
    state->tree_max_depth.entry = new Gtk_HEntryScale(&state->tree_max_depth);
    state->tree_ignore_depth.entry = 
	new Gtk_HEntryScale(&state->tree_ignore_depth);
    
    Gtk_VBox *tree_box = new Gtk_VBox(False, 0);
    tree_box->pack_start(*state->tree_center.box, TRUE, TRUE, 0);
    tree_box->pack_start(*state->tree_span.box, TRUE, TRUE, 0);
    notebook->append_page(*tree_box, *(new Gtk_Label("Size")));
    tree_box->show();
    state->tree_center.box->show();
    
    tree_box = new Gtk_VBox(False, 0);

    tree_count = tree_min_level = tree_max_level = poly_count = NULL;

    tree_min_level_string = tree_max_level_string = tree_count_string = 
	poly_count_string = NULL;

    update_stats();

    Gtk_Frame * stats_frame = new Gtk_Frame("Stats"); 
    Gtk_VBox * stats_box = new Gtk_VBox(False, 0);
    stats_box->pack_start(*poly_count, TRUE, TRUE, 0);
    stats_box->pack_start(*tree_count, TRUE, TRUE, 0);
    stats_box->pack_start(*tree_min_level, TRUE, TRUE, 0);
    stats_box->pack_start(*tree_max_level, TRUE, TRUE, 0);
    stats_frame->add(stats_box);

    tree_box->pack_start(*stats_frame, TRUE, TRUE, 4);
    tree_box->pack_start(*state->init_tree_depth.entry, TRUE, TRUE, 0);
    tree_box->pack_start(*state->tree_bisect_iterations.entry, TRUE, TRUE, 0);
    tree_box->pack_start(*state->tree_apply_weight.entry, TRUE, TRUE, 0);
    tree_box->pack_start(*state->tree_max_depth.entry, TRUE, TRUE, 0);
    tree_box->pack_start(*state->tree_ignore_depth.entry, TRUE, TRUE, 0);

    state->init_tree_depth.entry->show();
    state->tree_bisect_iterations.entry->show();
    state->tree_apply_weight.entry->show();
    state->tree_max_depth.entry->show();
    state->tree_ignore_depth.entry->show();

//    state->tree_ignore_depth.apply();

    notebook->append_page(*tree_box, *(new Gtk_Label("Depth")));

    stats_box->show();
    tree_box->show();
    stats_frame->show();
    tree_count->show();
    tree_min_level->show();
    tree_max_level->show();
    poly_count->show();


    h_func_box->show();
    func_box->show();

//     func_box->pack_start(func_label, TRUE, TRUE, 0);
//     func_box->pack_start(gen_func, TRUE, TRUE, 0);
    
//     func_box->set_spacing(2);

    func_label->show();
//     func_box->show();
    
//     Gtk_HBox* options_box = new Gtk_HBox(False);

//     options_box->pack_start(fract_label, 1, 1, 0);
//     options_box->pack_start(state_options, 1, 1, 0);
//     options_box->show();
    fract_label->show();

    apply_button->show();
    dismiss_button->show();

    Gtk_HSeparator * sep = new Gtk_HSeparator;
    //     top_box->pack_start(options_box, TRUE, TRUE, 2);
//     top_box->pack_start(func_box, TRUE, TRUE, 0);
    top_box->pack_start(*table, TRUE, TRUE, 2);
    top_box->pack_start(*sep, TRUE, TRUE, 2);
    top_box->pack_start(*notebook, TRUE, TRUE, 0);
    top_box->pack_start(*button_box, TRUE, TRUE, 0);
    sep->show();
    button_box->show();
    gen_func->show();
    printf("%d %d size\n", gen_func->width(), gen_func->height());
    //    handle->set_usize(gen_func->width(), gen_func->height());
    //    handle->show();
    printf("%d %d size\n", gen_func->width(), gen_func->height());
    //  notebook->set_show_border(4);
//    notebook->border_width(4);
    notebook->show();

    connect_to_method(use_this_c->toggled, state, 
		      &Fract_Params_State::toggle_using_c_from_const_box);
    connect_to_method(apply_button->clicked, this, &Fract_Params::apply);
    connect_to_signal(dismiss_button->clicked, hide);
    connect_to_method(show_bounding_box->toggled, this,
		      &Fract_Params::toggle_bounding_box);
//     connect_to_method(state->minval.box->changed, this,
// 		      &Fract_Params::change_zoom);
//     connect_to_method(state->maxval.box->changed, this,
// 		      &Fract_Params::change_zoom);
    connect_to_method(state->zoom.box->changed, this,
		      &Fract_Params::affect_zoom);
    connect_to_method(state->pan.box->changed, this,
		      &Fract_Params::affect_zoom);
//     connect_to_method(state->maxval.box->revert->clicked, this,
// 		      &Fract_Params::remove_zoom_box);
    connect_to_method(state->pan.box->revert->clicked, this,
		      &Fract_Params::remove_zoom_box);    connect_to_method(state->zoom.box->revert->clicked, this,
		      &Fract_Params::remove_zoom_box);
//     connect_to_method(state->minval.box->revert->clicked, this,
// 		      &Fract_Params::remove_zoom_box);
//    connect(gen_func->changed, this, &Fract_Params::change_expr);

    add(top_box);
    top_box->show();
    show();
    printf("%d %d size\n", gen_func->width(), gen_func->height());

}

void Fract_Params::update_stats() {
    int min_depth;
    int max_depth;
    int count;
    int num_polys;
    char ** f;
    
    if(state->fract && state->fract->tree) {
	max_depth = state->fract->tree->max_depth();
	min_depth = state->fract->tree->min_depth();
	count = state->fract->tree->count();
	num_polys = state->fract->params->poly_count();
    } else {
	min_depth = 0;
	max_depth = 0;
	count = 0;
	num_polys = 0;
    }

    char buf[1024];

    sprintf(buf, "Tree has %d nodes", count);
    if(tree_count_string)
	delete tree_count_string;
    tree_count_string = strdup(buf);
    if(tree_count)
	tree_count->set(tree_count_string);
    else
	tree_count = new Gtk_Label(tree_count_string);    

    sprintf(buf, "Minimum tree depth: %d", min_depth);
    if(tree_min_level_string)
	delete tree_min_level_string;
    tree_min_level_string = strdup(buf);
    if(tree_min_level) 
	tree_min_level->set(tree_min_level_string);
    else
	tree_min_level = new Gtk_Label(strdup(buf));

    sprintf(buf, "Maximum tree depth: %d", max_depth);
    if(tree_max_level_string)
	delete tree_max_level_string;
    if(tree_max_level)
	tree_max_level->set(strdup(buf));
    else
	tree_max_level = new Gtk_Label(strdup(buf));
    
    sprintf(buf, "Model has %d polygons", num_polys);
    if(poly_count_string)
	delete poly_count_string;
    if(poly_count)
	poly_count->set(strdup(buf));
    else
	poly_count = new Gtk_Label(strdup(buf));
}

void Fract_Params::remove_zoom_box() {
    state->zoom_box_visible = false;
    if(state->fract)
	state->fract->refresh();
}

void Fract_Params::change_expr() {

//     if(strchr(gen_func->get_text(),'\n'))
// 	fract->set_expr(gen_func->get_text());
}

void Fract_Params::affect_zoom() {
    
    if(!state->zoom_box_visible)
	state->zoom_box_visible = true;
    
    if(state->fract)
	state->fract->request(FRACT_REFRESH_REQ);
}

void Fract_Params::change_zoom() {

    if(!state->zoom_box_visible)
	state->zoom_box_visible = true;

    if(state->fract)
	state->fract->request(FRACT_REFRESH_REQ);
}

void Fract_Params::toggle_bounding_box() {

    state->bounding_box_visible = !state->bounding_box_visible;
    if(state->fract)
	state->fract->request(FRACT_REFRESH_REQ);    
}

void Fract_Params::append_fractal(fractal_State *s) {
    Gtk_MenuItem *item = new Gtk_MenuItem(s->name);
    state_menu->append(*item);
    item->show();
}
