/*
 *  fract_params.cc
 * 
 *  Fractal Generator
 * 
 *  Copyright (c) 1998 Brian Martin
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <math.h>
#include "fract_params.h"
#include "fractal.h"
#include "defs.h"
#include "statelist.h"

extern StateList fractals;
extern struct yy_buffer_state * yy_scan_string(const char *);

				// updates fractal params from gui
void Fract_Params::apply() {

    bool need_recompute_sizechange = false;
    bool need_recompute = false;
    bool need_refresh = false;

    if(strcmp(gen_func->get_text(), state->expr_string)!=0) {
	state->set_expr(gen_func->get_text());
	need_recompute = true;
	printf("expr_set\n");
    }

    if(state->minval.changed()) {
	need_recompute = true; 
	state->minval.grab();
    }

    if(state->w_fix.changed()) {
	state->w_fix.grab();
	need_recompute = true;
    }
    
    if(state->maxval.changed()) {
	need_recompute = true;
	state->maxval.grab();
    }

    if(state->zoom.changed()) {
	need_recompute = true;
	state->zoom.apply();
    }

    if(state->pan.changed()) {
	need_recompute = true;
	state->pan.apply();
    }

    if(state->size.changed()) {	
	need_recompute_sizechange = true;
//	state->size.grab();
	printf("size changed\n");
    } else 
	printf("size not changed\n");

    if(state->bounding_res.changed()) {
	need_refresh = true;
	state->bounding_res.grab();
    }

    printf("queueing requests\n");

    if(need_recompute || need_recompute_sizechange)
	state->bounding_box_visible = false;

    if(need_recompute_sizechange)
	state->fract->request(FRACT_RECOMPUTE_SIZECHANGE_REQ);

    printf("sizechange\n");
    
    if(need_recompute && !need_recompute_sizechange)
	state->fract->request(FRACT_RECOMPUTE_REQ);
    
    printf("recompute\n");

    if(need_recompute || need_refresh)
	state->fract->request(FRACT_REFRESH_REQ);    

    printf("requests queued\n");
}

Fract_Params::Fract_Params(Fract_Params_State * _state):
    state(_state)
{
    notebook = new Gtk_Notebook;
    gen_func = new Gtk_Entry;
    
    top_box = new Gtk_VBox(False, 0);
    size_box = new Gtk_VBox(False, 0);
    val_box = new Gtk_VBox(False, 0);
    misc_box = new Gtk_VBox(False, 0);
    zoom_box = new Gtk_VBox(False, 0);
    Gtk_VBox * func_box = new Gtk_VBox(False, 0);

//    func_box = new Gtk_HBox(False, 4);
    
    func_label = new Gtk_Label("f(q): ");

    state_options = new Gtk_OptionMenu;
    state_menu = new Gtk_Menu;
    Gtk_MenuItem * item;

    for(GList * ptr = fractals.get_list() ; ptr ; ptr = ptr->next) 
	if(ptr->data) {
	    item = new Gtk_MenuItem(((fractal_State*)ptr->data)->name);
	    state_menu->append(item);
	    item->show();
	}

    connect_to_method(fractals.append, this, &Fract_Params::append_fractal);

    state_options->set_menu(state_menu);
    state_options->show();

    state->minval.box  = new Gtk_CoordBox(&state->minval);
    state->maxval.box = new Gtk_CoordBox(&state->maxval);
    state->size.box = new Gtk_CoordBox(&state->size);
    state->zoom.box = new Gtk_CoordBox(&state->zoom);
    state->pan.box = new Gtk_CoordBox(&state->pan);
    state->w_fix.entry = new Gtk_HEntryScale(&state->w_fix);

    button_box = new Gtk_HBox(False, 0);
    dismiss_button = new Gtk_Button("dismiss");
    apply_button = new Gtk_Button("apply");

    bounding_frame = new Gtk_Frame("Bounding Box");
    show_bounding_box = new Gtk_CheckButton("show");
    bounding_box = new Gtk_VBox(False, 0);
    state->bounding_res.entry = new Gtk_HEntryScale(&state->bounding_res);
    
    bounding_box->pack_start(state->bounding_res.entry, TRUE, TRUE, 4);
    bounding_box->pack_start(show_bounding_box, TRUE, TRUE, 4);
    show_bounding_box->show();
    state->bounding_res.entry->show();
    bounding_frame->add(bounding_box);
    bounding_box->show();

//    misc_box->pack_start(gen_func, TRUE, TRUE, 4);
    misc_box->pack_start(state->w_fix.entry, TRUE, TRUE, 4);
    misc_box->pack_start(bounding_frame, TRUE, TRUE, 4);
    val_box->pack_start(state->minval.box, TRUE, TRUE, 4);
    val_box->pack_start(state->maxval.box, TRUE, TRUE, 4);
    size_box->pack_start(state->size.box, TRUE, TRUE, 4);
    size_box->pack_start(misc_box, TRUE, TRUE, 4);
    zoom_box->pack_start(state->zoom.box, TRUE, TRUE, 4);
    zoom_box->pack_start(state->pan.box, TRUE, TRUE, 4);
//    size_box->pack_start(val_box, TRUE, TRUE, 4);
    notebook->append_page(val_box, new Gtk_Label("Bounds"));
    notebook->append_page(size_box, new Gtk_Label("Size"));
    notebook->append_page(zoom_box, new Gtk_Label("Zoom"));
//    notebook->append_page(NULL, new Gtk_Label("Colors"));

    size_box->show();
    bounding_frame->show();
    misc_box->show();
    val_box->show();
    zoom_box->show();

    state->w_fix.entry->show();
    state->size.box->show();
    state->minval.box->show();
    state->maxval.box->show();
    state->zoom.box->show();
    state->pan.box->show();

    button_box->pack_start(apply_button, TRUE, TRUE, 0);
    button_box->pack_start(dismiss_button, TRUE, TRUE, 0);

    GTK_WIDGET_SET_FLAGS(apply_button->gtkobject, GTK_CAN_DEFAULT);
    GTK_WIDGET_SET_FLAGS(dismiss_button->gtkobject, GTK_CAN_DEFAULT);

    gen_func->set_text(state->expr_string);

    Gtk_Label * fract_label = new Gtk_Label("Fractal:");
    Gtk_Table * table = new Gtk_Table(1, 4, 1);
    table->attach_defaults(fract_label, 0, 1, 0, 1);
    table->attach_defaults(state_options, 1, 4, 0, 1);
    table->show();

    Gtk_HBox * h_func_box = new Gtk_HBox(False, 0);
    h_func_box->pack_start(func_label, TRUE, TRUE, 4);
    h_func_box->pack_start(gen_func, TRUE, TRUE, 4);
    func_box->pack_start(h_func_box, TRUE, TRUE, 0);

    Gtk_CheckButton *q_is_c_initially = 
	new Gtk_CheckButton("Q is C initially");
    
    func_box->pack_start(q_is_c_initially, TRUE, TRUE, 0);
    q_is_c_initially->show();

    notebook->append_page(func_box, new Gtk_Label("Function"));

    h_func_box->show();
    func_box->show();

//     func_box->pack_start(func_label, TRUE, TRUE, 0);
//     func_box->pack_start(gen_func, TRUE, TRUE, 0);
    
//     func_box->set_spacing(2);

    func_label->show();
//     func_box->show();
    
//     Gtk_HBox* options_box = new Gtk_HBox(False);

//     options_box->pack_start(fract_label, 1, 1, 0);
//     options_box->pack_start(state_options, 1, 1, 0);
//     options_box->show();
    fract_label->show();

    apply_button->show();
    dismiss_button->show();

    Gtk_HSeparator * sep = new Gtk_HSeparator;
    //     top_box->pack_start(options_box, TRUE, TRUE, 2);
//     top_box->pack_start(func_box, TRUE, TRUE, 0);
    top_box->pack_start(table, TRUE, TRUE, 2);
    top_box->pack_start(sep, TRUE, TRUE, 2);
    top_box->pack_start(notebook, TRUE, TRUE, 0);
    top_box->pack_start(button_box, TRUE, TRUE, 0);
    sep->show();
    button_box->show();
    gen_func->show();
    //  notebook->set_show_border(4);
//    notebook->border_width(4);
    notebook->show();

    connect_to_method(q_is_c_initially->toggled, this,
		      &Fract_Params::toggle_init_q);
    connect_to_method(apply_button->clicked, this, &Fract_Params::apply);
    connect_to_signal(dismiss_button->clicked, hide);
    connect_to_method(show_bounding_box->toggled, this,
		      &Fract_Params::toggle_bounding_box);
    connect_to_method(state->minval.box->changed, this,
		      &Fract_Params::change_zoom);
    connect_to_method(state->maxval.box->changed, this,
		      &Fract_Params::change_zoom);
    connect_to_method(state->zoom.box->changed, this,
		      &Fract_Params::affect_zoom);
    connect_to_method(state->pan.box->changed, this,
		      &Fract_Params::affect_zoom);
    connect_to_method(state->maxval.box->revert->clicked, this,
		      &Fract_Params::remove_zoom_box);
    connect_to_method(state->pan.box->revert->clicked, this,
		      &Fract_Params::remove_zoom_box);
    connect_to_method(state->zoom.box->revert->clicked, this,
		      &Fract_Params::remove_zoom_box);
    connect_to_method(state->minval.box->revert->clicked, this,
		      &Fract_Params::remove_zoom_box);
//    connect(gen_func->changed, this, &Fract_Params::change_expr);

    add(top_box);
    top_box->show();
    show();
}

void Fract_Params::toggle_init_q() {
    state->q_is_c_initially = !state->q_is_c_initially;
}

void Fract_Params::remove_zoom_box() {
    state->zoom_box_visible = false;
    state->fract->refresh();
}

void Fract_Params::change_expr() {

//     if(strchr(gen_func->get_text(),'\n'))
// 	fract->set_expr(gen_func->get_text());
}

void Fract_Params::affect_zoom() {
    
    if(!state->zoom_box_visible)
	state->zoom_box_visible = true;

    double min_x = state->minval.x.value();
    double min_y = state->minval.y.value();
    double min_z = state->minval.z.value();

    double max_x = state->maxval.x.value();
    double max_y = state->maxval.y.value();
    double max_z = state->maxval.z.value();

    double x_mid = (max_x+min_x)/2;
    double y_mid = (max_y+min_y)/2;
    double z_mid = (max_z+min_z)/2;

    double x_span = max_x-min_x;
    double y_span = max_y-min_y;
    double z_span = max_z-min_z;

    x_span *= state->zoom.x.gui_value();
    y_span *= state->zoom.y.gui_value();
    z_span *= state->zoom.z.gui_value();
    
    x_span *= 0.5;
    y_span *= 0.5;
    z_span *= 0.5;

    x_mid += state->pan.x.gui_value();
    y_mid += state->pan.y.gui_value();
    z_mid += state->pan.z.gui_value();

    state->maxval.x.entry->setVal(x_mid+x_span);
    state->maxval.y.entry->setVal(y_mid+y_span);
    state->maxval.z.entry->setVal(z_mid+z_span);
    state->minval.x.entry->setVal(x_mid-x_span);
    state->minval.y.entry->setVal(y_mid-y_span);
    state->minval.z.entry->setVal(z_mid-z_span);
    
    state->fract->request(FRACT_REFRESH_REQ);    
}

void Fract_Params::change_zoom() {

    if(!state->zoom_box_visible)
	state->zoom_box_visible = true;

    state->fract->request(FRACT_REFRESH_REQ);
}

void Fract_Params::toggle_bounding_box() {

    state->bounding_box_visible = !state->bounding_box_visible;
    state->fract->request(FRACT_REFRESH_REQ);    
}

void Fract_Params::append_fractal(fractal_State *s) {
    Gtk_MenuItem *item = new Gtk_MenuItem(s->name);
    state_menu->append(item);
    item->show();
}
