/*
 *  fract_params.h
 * 
 *  Fractal Generator
 * 
 *  Copyright (c) 1998 Brian Martin
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef FRACT_PARAMS_H
#define FRACT_PARAMS_H

/*
  this file holds all the fractal params

  bounds, resolution, w fix, generator function...

 */

#include <gtk--.h>
#include <string.h>
#include <libquat/expr_tree.h>
#include "state.h"
#include "Gtk_CoordBox.h"
#include "Gtk_HEntryScale.h"
#include "fractal_state.h"
#include "fract_params_state.h"

class Fract_Params : public Gtk_Window {

private:

    Gtk_Notebook *notebook;

    Gtk_OptionMenu * state_options;
    Gtk_Menu * state_menu;

    Gtk_HBox *func_box;
    Gtk_Label *func_label;

    Gtk_VBox *top_box;		// top level box

    Gtk_VBox *zoom_box;		// zoom controls
    Gtk_VBox *misc_box;		// holds W fix and bounding stuff

    Gtk_Frame *bounding_frame;
    Gtk_VBox *bounding_box;

    Gtk_HBox *button_box;
    Gtk_Button *dismiss_button;
    Gtk_Button *apply_button;

    Gtk_Label * tree_count;
    Gtk_Label * tree_min_level;
    Gtk_Label * tree_max_level;
    Gtk_Label * poly_count;

    char * tree_count_string;
    char * tree_min_level_string;
    char * tree_max_level_string;        
    char * poly_count_string;

//    class fractal * fract;

    void change_zoom();
    void affect_zoom();
    void toggle_bounding_box();
    void change_expr();

public:

    void update_stats();

    Gtk_CheckButton *show_bounding_box;

//    Gtk_Combo *gen_func;

    Gtk_Entry *constant_s, *constant_i, *constant_j, *constant_k;

    Gtk_Entry *gen_func;
    Gtk_Entry *init_func;
    class Fract_Params_State * state;

    void append_fractal(fractal_State*);

    Fract_Params(Fract_Params_State * _state);
    void remove_zoom_box();
    //    void reset_bounds();
    void apply();
    expr_ref set_expr(const char *);
    expr_ref set_expr(FILE *);    
};

#endif
