/*
 *  fract_params_state.cc
 * 
 *  Fractal Generator
 * 
 *  Copyright (c) 1998 Brian Martin
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "fract_params.h"
#include "fractal.h"

extern struct yy_buffer_state * yy_scan_string(const char *);
extern int yyparse();

Fract_Params_State::Fract_Params_State() {
    add_child(&tree_center);
    add_child(&tree_span);
    add_child(&init_tree_depth);
    add_child(&tree_bisect_iterations);
    add_child(&tree_apply_weight);
    add_child(&tree_max_depth);
    add_child(&tree_ignore_depth);
    add_child(&w_fix);
    add_child(&bounding_res);
    add_child(&zoom);
    add_child(&pan);
    add_child(&maxiter);
    fract = NULL;
    gui = NULL;
}

void Fract_Params_State::load(FILE * fp) {
    char buf[200];
    
    this->::State::load(fp);
    
    fscanf(fp, "%d %d %d\n",
	   &bounding_box_visible,
	   &zoom_box_visible);

    set_expr(fp);

//     if(fract)
// 	fract->set_table(expr_tree);

    fscanf(fp, " %lf %lf %lf %lf ", // make sure this is right
	   initial_q_val, 
	   initial_q_val+1,
	   initial_q_val+2,
	   initial_q_val+3);
}

void Fract_Params_State::save(FILE * fp) {

    ::State::save(fp);

    fprintf(fp, " %d %d %d ",
	    bounding_box_visible,
	    zoom_box_visible);

    if(expr_string)
	fprintf(fp, "%s\n", expr_string);
    else
	fprintf(fp, "%s\n", expr_tree->make_string());

    for(int i = 0 ; i < 4 ; i++) 
	fprintf(fp, "%.5f ", initial_q_val[i]);
    fprintf(fp, "\n");
}

Fract_Params * Fract_Params_State::show_dialog() { 
    return gui = new Fract_Params(this); 
}

extern expr_ref tree_top;
	    
void Fract_Params_State::set_expr(const char * expression, const char * init) {

    if(gui && use_c_from_const_box) {
	char buf[1024];
	sprintf(buf, "[ %s %s %s %s ]",
		gui->constant_s->get_text().c_str(),
		gui->constant_i->get_text().c_str(),
		gui->constant_j->get_text().c_str(),
		gui->constant_k->get_text().c_str());
	printf("[ %.20f %.20f %.20f %.20f ]",
	       gui->constant_s->get_text().c_str(),
	       gui->constant_i->get_text().c_str(),
	       gui->constant_j->get_text().c_str(),
	       gui->constant_k->get_text().c_str());
	char * needle = strchr(expression, 'c');
	if(needle) {		// substitute constant
	    char end_buf[2048];
	    strncpy(end_buf, expression, needle-expression);
	    strcpy(end_buf+(needle-expression), buf);
	    strcat(end_buf, needle+1);
	    expression = end_buf;
	}
    }
    printf("expr.. %s\n", expression);

    expr_string = strdup(expression);// should maybe free this at some point
    init_expr_string = strdup(init);
    
    if(gui) {
      if(strcmp(expr_string,(gui->gen_func->get_text().c_str()))!=0)
	gui->gen_func->set_text(expr_string);
      if(strcmp(init_expr_string,(gui->init_func->get_text().c_str()))!=0)
	gui->init_func->set_text(init_expr_string);
    }
}

extern FILE * yyin;

void Fract_Params_State::set_expr(FILE * fp) {

    expr_ref expr;

    char buf[1024];

    fgets(buf, 1024, fp);

    //    set_expr(buf);

}
 
void Fract_Params_State::toggle_using_c_from_const_box() {
    use_c_from_const_box = !use_c_from_const_box;
}
	
void Fract_Params_State::set_default() {

  //    expr_list = set_expr("e^(i*pi/2)*(q^2-[0.328591 0.38861 0.0 0.0])");
    
    set_expr("q^2+c", "c");//+[-0.64447472536401099362 -0.4397751611159561619 0 0]");
//    set_expr("(|re(q)|+i*|im_i(q)|+j*|im_j(q)|+k*|im_k(q)|)^2+c");
				// maybe we can get rid of expr_list ?
//-0.73835117674326855775 -0.13577596886179760814 0 0]");
//

    use_c_from_const_box = false;

    const_s = "0.0";
    const_a = "0.0";
    const_b = "0.0";
    const_c = "0.0";

    bounding_box_visible = false;
    zoom_box_visible = false;

    initial_q_val[0] = 0;
    initial_q_val[1] = 0;
    initial_q_val[2] = 0;
    initial_q_val[3] = 0;
    
    w_fix.label = "W_fix:";
    w_fix.minval = -3;
    w_fix.maxval = 3;
    w_fix.val = 0;
    w_fix.res = 0.1;
    w_fix.small_step = 0.1;

    bounding_res.label = "Boundres";
    bounding_res.val = 1.0;
    bounding_res.minval = 1;
    bounding_res.maxval = 9;
    bounding_res.res = 1.0;
    bounding_res.small_step = 1.0;

    maxiter.label = "maxiter";
    maxiter.val = 20.0;
    maxiter.minval = 1;
    maxiter.maxval = 500;
    maxiter.res = 0.01;
    maxiter.small_step = 0.5;
    maxiter.log = 1;

    zoom.label = "zoom";

    zoom.x.label = "X:";
    zoom.y.label = "Y:";
    zoom.z.label = "Z:";

    zoom.x.minval = 0.01;
    zoom.y.minval = 0.01;
    zoom.z.minval = 0.01;

    zoom.x.maxval = 100.0;
    zoom.y.maxval = 100.0;
    zoom.z.maxval = 100.0;

    zoom.x.val = 1;
    zoom.y.val = 1;
    zoom.z.val = 1;

    zoom.x.res = .001;
    zoom.y.res = .001;
    zoom.z.res = .001;

    zoom.x.small_step = .005;
    zoom.y.small_step = .005;
    zoom.z.small_step = .005;

    zoom.x.log = 1;
    zoom.y.log = 1;
    zoom.z.log = 1;

    pan.label = "pan";

    pan.x.label = "X:";
    pan.y.label = "Y:";
    pan.z.label = "Z:";

    pan.x.minval = -2;
    pan.y.minval = -2;
    pan.z.minval = -2;

    pan.x.maxval = 2.0;
    pan.y.maxval = 2.0;
    pan.z.maxval = 2.0;

    pan.x.val = 0;
    pan.y.val = 0;
    pan.z.val = 0;

    pan.x.res = .001;
    pan.y.res = .001;
    pan.z.res = .001;

    pan.x.small_step = .005;
    pan.y.small_step = .005;
    pan.z.small_step = .005;

    pan.x.log = 0;
    pan.y.log = 0;
    pan.z.log = 0;

    tree_center.label = "center";

    tree_center.x.label = "X:";
    tree_center.y.label = "Y:";
    tree_center.z.label = "Z:";

    tree_center.x.minval = -2;
    tree_center.y.minval = -2;
    tree_center.z.minval = -2;

    tree_center.x.maxval = 2.0;
    tree_center.y.maxval = 2.0;
    tree_center.z.maxval = 2.0;

    tree_center.x.val = 0;
    tree_center.y.val = 0;
    tree_center.z.val = 0;

    tree_center.x.res = .001;
    tree_center.y.res = .001;
    tree_center.z.res = .001;

    tree_center.x.small_step = .005;
    tree_center.y.small_step = .005;
    tree_center.z.small_step = .005;

    tree_center.x.log = 0;
    tree_center.y.log = 0;
    tree_center.z.log = 0;

    tree_span.label = "span";

    tree_span.x.label = "X:";
    tree_span.y.label = "Y:";
    tree_span.z.label = "Z:";

    tree_span.x.minval = 0;
    tree_span.y.minval = 0;
    tree_span.z.minval = 0;

    tree_span.x.maxval = 8.0;
    tree_span.y.maxval = 8.0;
    tree_span.z.maxval = 8.0;

    tree_span.x.val = 4;
    tree_span.y.val = 4;
    tree_span.z.val = 4;

    tree_span.x.res = .001;
    tree_span.y.res = .001;
    tree_span.z.res = .001;

    tree_span.x.small_step = .005;
    tree_span.y.small_step = .005;
    tree_span.z.small_step = .005;

    tree_span.x.log = 0;
    tree_span.y.log = 0;
    tree_span.z.log = 0;

    init_tree_depth.label = "init depth:";
    init_tree_depth.minval = 0;
    init_tree_depth.maxval = 9;
    init_tree_depth.val = 2;
    init_tree_depth.res = 1.0;
    init_tree_depth.small_step = 1.0;
    init_tree_depth.log = 0;

    tree_bisect_iterations.label = "bisect iter:";
    tree_bisect_iterations.minval = 0;
    tree_bisect_iterations.maxval = 20;
    tree_bisect_iterations.val = 10;
    tree_bisect_iterations.res = 1.0;
    tree_bisect_iterations.small_step = 1.0;
    tree_bisect_iterations.log = 0;

    tree_apply_weight.label = "apply depth:";
    tree_apply_weight.minval = 0;
    tree_apply_weight.maxval = 9;
    tree_apply_weight.val = 1;
    tree_apply_weight.res = 1.0;
    tree_apply_weight.small_step = 1.0;
    tree_apply_weight.log = 0;

    tree_max_depth.label = "max depth:";
    tree_max_depth.minval = 0;
    tree_max_depth.maxval = 100;
    tree_max_depth.val = 50;
    tree_max_depth.res = 1.0;
    tree_max_depth.small_step = 1.0;
    tree_max_depth.log = 0;

    tree_ignore_depth.label = "empty depth:";
    tree_ignore_depth.minval = 0;
    tree_ignore_depth.maxval = 5;
    tree_ignore_depth.val = 2;
    tree_ignore_depth.res = 1.0;
    tree_ignore_depth.small_step = 1.0;
    tree_ignore_depth.log = 0;
}

void Fract_Params_State::grab() {

    ::State::grab();

    expr_string = NULL;	// need to check if gui has changed
    expr_list = NULL;
	    	    
    bounding_box_visible = gui->show_bounding_box->get_mode();

}

void Fract_Params_State::apply() {
    ::State::apply();
//	    gui->show_bounding_box->
}
bool Fract_Params_State::changed() {
    return(w_fix.changed() || bounding_res.changed());
}
