/*
 *  fract_params_state.cc
 * 
 *  Fractal Generator
 * 
 *  Copyright (c) 1998 Brian Martin
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "fract_params.h"
#include "fractal.h"

extern struct yy_buffer_state * yy_scan_string(const char *);
extern int yyparse();

Fract_Params_State::Fract_Params_State() {
    add_child(&size);
    add_child(&w_fix);
    add_child(&minval);
    add_child(&maxval);
    add_child(&bounding_res);
    add_child(&zoom);
    add_child(&pan);
}

void Fract_Params_State::load(FILE * fp) {
    char buf[200];
    
    this->::State::load(fp);
    
    fscanf(fp, "%d %d %d\n",
	   &bounding_box_visible,
	   &zoom_box_visible,
	   &q_is_c_initially);

    set_expr(fp);

    if(fract)
	fract->set_table(expr_tree);

    fscanf(fp, " %lf %lf %lf %lf ", // make sure this is right
	   initial_q_val, 
	   initial_q_val+1,
	   initial_q_val+2,
	   initial_q_val+3);
}

void Fract_Params_State::save(FILE * fp) {

    ::State::save(fp);

    fprintf(fp, " %d %d %d ",
	    bounding_box_visible,
	    zoom_box_visible,
	    q_is_c_initially);

    if(expr_string)
	fprintf(fp, "%s\n", expr_string);
    else
	fprintf(fp, "%s\n", expr_tree->make_string());

    for(int i = 0 ; i < 4 ; i++) 
	fprintf(fp, "%.5f ", initial_q_val[i]);
    fprintf(fp, "\n");
}

void Fract_Params_State::show_dialog() { gui = new Fract_Params(this); }

extern expr_ref tree_top;
	    
expr_ref Fract_Params_State::set_expr(const char * expression) {

    void * buf = (void *)yy_scan_string(expression);
    expr_ref expr;
    expr_string = strdup(expression); // maybe this should strdup?
    
    if(yyparse() == 0) {
	expr_tree = tree_top;
//	expr_list = expr;

	if(gui && strcmp(expr_string,gui->gen_func->get_text())!=0)
	    gui->gen_func->set_text(expr_string);
    
	if(fract)
	    fract->set_table(tree_top);

	return expr;
    } else
	return 0;
    
}

extern FILE * yyin;

expr_ref Fract_Params_State::set_expr(FILE * fp) {

    expr_ref expr;

    char buf[1024];

    fgets(buf, 1024, fp);

    set_expr(buf);

//     (void *)yy_scan_string(buf);

//     if(yyparse() == 0)
// 	expr = tree_top->tree2list((expr_ref)NULL);    
//     else 
// 	expr = NULL;

//     expr_tree = tree_top;
//     expr_list = expr;
//     delete expr_string;
//     expr_string = strdup(buf);	// should probably free previous string
				
//     if(gui && strcmp(expr_string,gui->gen_func->get_text())!=0)
// 	gui->gen_func->set_text(expr_string);

//     if(fract)
// 	fract->set_table(tree_top);

//     return expr;
}
	
void Fract_Params_State::set_default() {

    expr_list = set_expr("e^(i*pi/2)*(q^2-[0.328591 0.38861 0.0 0.0])");
    if(fract)
	fract->set_table(expr_tree);
	    
    bounding_box_visible = false;
    zoom_box_visible = false;
    q_is_c_initially = true;

    initial_q_val[0] = 0;
    initial_q_val[1] = 0;
    initial_q_val[2] = 0;
    initial_q_val[3] = 0;
    
    w_fix.label = "W_fix:";
    w_fix.minval = -3;
    w_fix.maxval = 3;
    w_fix.val = 0;
    w_fix.res = 0.1;

    bounding_res.label = "Boundres";
    bounding_res.val = 1.0;
    bounding_res.minval = 1;
    bounding_res.maxval = 9;
    bounding_res.res = 1.0;

    minval.label = "minvals";

    minval.x.label = "X:";
    minval.y.label = "Y:";
    minval.z.label = "Z:";

    maxval.x.label = "X:";
    maxval.y.label = "Y:";
    maxval.z.label = "Z:";
	    
    minval.x.minval = -3.0;
    minval.y.minval = -3.0;
    minval.z.minval = -3.0;

    minval.x.maxval = 3.0;
    minval.y.maxval = 3.0;
    minval.z.maxval = 3.0;

    minval.x.log = 0;
    minval.y.log = 0;
    minval.z.log = 0;

    maxval.label = "maxvals";

    maxval.x.maxval = 3.0;
    maxval.y.maxval = 3.0;
    maxval.z.maxval = 3.0;

    maxval.x.minval = -3.0;
    maxval.y.minval = -3.0;
    maxval.z.minval = -3.0;

    minval.x.val = -2.0;
    minval.y.val = -2.0;
    minval.z.val = -2.0;

    maxval.x.val = 2.0;
    maxval.y.val = 2.0;
    maxval.z.val = 2.0;

    minval.x.res = 0.1;
    minval.y.res = 0.1;
    minval.z.res = 0.1;

    maxval.x.res = 0.1;
    maxval.y.res = 0.1;
    maxval.z.res = 0.1;

    maxval.x.log = 0;
    maxval.y.log = 0;
    maxval.z.log = 0;
	    
    size.label = "size";

    size.x.label = "X:";
    size.y.label = "Y:";
    size.z.label = "Z:";

    size.x.minval = 1;
    size.y.minval = 1;
    size.z.minval = 1;

    size.x.maxval = 200;
    size.y.maxval = 200;
    size.z.maxval = 200;
	    
    size.x.val = 20;
    size.y.val = 20;
    size.z.val = 20;
	    
    size.x.res = 1.0;
    size.y.res = 1.0;
    size.z.res = 1.0;

    size.x.log = 0;
    size.y.log = 0;
    size.z.log = 0;

    zoom.label = "zoom";

    zoom.x.label = "X:";
    zoom.y.label = "Y:";
    zoom.z.label = "Z:";

    maxval.x.label = "X:";
    maxval.y.label = "Y:";
    maxval.z.label = "Z:";
	    
    zoom.x.minval = 0.1;
    zoom.y.minval = 0.1;
    zoom.z.minval = 0.1;

    zoom.x.maxval = 10.0;
    zoom.y.maxval = 10.0;
    zoom.z.maxval = 10.0;

    zoom.x.val = 1;
    zoom.y.val = 1;
    zoom.z.val = 1;

    zoom.x.res = .001;
    zoom.y.res = .001;
    zoom.z.res = .001;

    zoom.x.log = 1;
    zoom.y.log = 1;
    zoom.z.log = 1;

    pan.label = "pan";

    pan.x.label = "X:";
    pan.y.label = "Y:";
    pan.z.label = "Z:";

    maxval.x.label = "X:";
    maxval.y.label = "Y:";
    maxval.z.label = "Z:";
	    
    pan.x.minval = -2;
    pan.y.minval = -2;
    pan.z.minval = -2;

    pan.x.maxval = 2.0;
    pan.y.maxval = 2.0;
    pan.z.maxval = 2.0;

    pan.x.val = 0;
    pan.y.val = 0;
    pan.z.val = 0;

    pan.x.res = .001;
    pan.y.res = .001;
    pan.z.res = .001;

    pan.x.log = 0;
    pan.y.log = 0;
    pan.z.log = 0;
}

void Fract_Params_State::grab() {

    ::State::grab();

    expr_string = NULL;	// need to check if gui has changed
    expr_list = NULL;
	    	    
    bounding_box_visible = gui->show_bounding_box->get_state();

}

void Fract_Params_State::apply() {
    ::State::apply();
//	    gui->show_bounding_box->
}
bool Fract_Params_State::changed() {
    return(minval.changed() || maxval.changed() || size.changed() ||
	   w_fix.changed() || bounding_res.changed());
}
