/*
 *  fract_view.cc
 * 
 *  Fractal Generator
 * 
 *  Copyright (c) 1998 Brian Martin
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "fract_view.h"
#include "fractal.h"
#include "defs.h"
#include "math.h"

Fract_View::Fract_View(Fract_View_State * _state) {

    state = _state;
    vbox = new Gtk_VBox(False,0);
    perspective = new Gtk_CheckButton("Perspective Mode");
    hbox = new Gtk_HBox(False,0);
    printf("min_zoom = %.2f max_zoom = %.2f zoom_val = %.2f\n",
	   state->min_zoom, state->max_zoom, state->zoom_val);
    zoom_adj = new Gtk_Adjustment(log(state->zoom_val),
				  log(state->min_zoom),
				  log(state->max_zoom),
				  0.1, 0.1, 0.1);
    zoom = new Gtk_VScale(*zoom_adj);
    rot = new Positioner(100,100);

//        rot.set_usize(100,100);
    
    state->view = this;

    zoom_val = 1;

    hbox->add(rot);
    hbox->add(zoom);
    vbox->add(hbox);
    vbox->add(perspective);
    gtk_widget_set_events(GTK_WIDGET(rot->gtkobj()),
			  GDK_BUTTON_PRESS_MASK | 
			  GDK_BUTTON_RELEASE_MASK |
			  GDK_EXPOSURE_MASK | 
			  GDK_BUTTON_MOTION_MASK);
    rot->show();
    zoom->show();
    perspective->show();
    vbox->show();
    hbox->show();
    add(vbox);

    state->rot.pos = rot;

    connect_to_method(perspective->toggled, this, 
		      &Fract_View::set_perspective);

    if(state->fract) {
	connected = true;
	printf("state is %p in view\n", state->fract);
	attach_signals(&(state->fract->refresh));
    } else {
	connected = false;
    }
}

void Fract_View::detach_signals() {
    if(connected) {
	rot_sig.disconnect();
	persp_sig.disconnect();
	zoom_sig.disconnect();
	connected = false;
    }
}

void Fract_View::attach_signals(Signal0 *signal) {
    connected = true;
    rot_sig = connect_to_signal(rot->changed, *signal);
    persp_sig = connect_to_signal(perspective->toggled, *signal);
    zoom_sig = connect_to_signal(zoom_adj->value_changed, *signal);
}

void Fract_View::set_zoom() { 

//    state->zoom_val = GTK_ADJUSTMENT(zoom_adj.gtkobject)->value;
//    printf("zooming to %.2f\n", GTK_ADJUSTMENT(zoom_adj->gtkobject)->value);
//    fract->refresh();

}

void Fract_View::set_perspective() {
    
    state->perspective_on = !state->perspective_on;

}
