/*
 *  fractal.cc
 * 
 *  Fractal Generator
 * 
 *  Copyright (c) 1998 Brian Martin
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "fractal.h"
#include "fractal_state.h"
#include "defs.h"
#include <time.h>
#include <math.h>

extern char * yy_scan_string(const char *);

#ifdef USE_THREADS

void *processes_requests(void * fract) {

    ((fractal*)fract)->processes_requests();
    pthread_exit(0);		// probably not necessary
}

void fractal::processes_requests() {
    while(1) {
	switch(requests.remove()) { // queue is synchronized
 	case FRACT_RECOMPUTE_REQ:
	    printf("recompute_req\n");
 	    recompute(false);
	    printf("recompute done\n");
 	    break;

	case FRACT_RECOMPUTE_SIZECHANGE_REQ:
	    printf("recompute sizechange\n");
	    recompute(true);
	    break;

 	case FRACT_REFRESH_REQ:
//	    sem_wait(&x_mutex);
//	    XLockDisplay(dpy);
	    printf("processing refresh\n");
	    requests.clear_pending(FRACT_REFRESH_REQ);	
	    refresh_impl();
	    printf("refresh done\n");
//	    XUnlockDisplay(dpy);
//	    sem_post(&x_mutex);
 	    break;
 	}
    }
}

#endif // USE_THREADS

fractal::fractal(fractal_State * _state) : 
#ifdef USE_THREADS    
    requests(200),
#endif
    state(_state)
{ 
    
    max_val = 20;

    tree = NULL;

    params = NULL;
    have_params = false;
#if 0
// USE_THREADS
    num_threads = 1;		// should probably parameterize this sometime..

    table = new eq_table*[num_threads];
    table[0] = new eq_table(state->gui.expr_tree, state->maxiter.gui_value(), max_val);
    for(int i = 1 ; i < num_threads ; i++) 
	//	table[i] = new eq_table(table[0]);
	table[i] = new eq_table(state->gui.expr_tree, state->maxiter.gui_value(), max_val);
#else
    table = NULL;
//    table = new eq_table(state->gui.expr_tree, state->maxiter.gui_value(), max_val);
#endif

    diverg_cache = new Hash<bool>(4130,4*sizeof(double));
    
    glwin = new GLPort(300, 300, state->filename);
    glwin->show();

    dpy = glwin->getDpy();
    drawable = glwin->getWin();

    glEnable(GL_VERTEX_ARRAY);
    glDisable(GL_NORMALIZE);	// we send unit normals for efficiency
				// or at least is seems like we should
				// but the lighting looks real bad when we do.
    glDepthFunc(GL_LESS);	// enable zbuffer
    glEnable(GL_DEPTH_TEST);
    glEnable(GL_COLOR_MATERIAL);
    glShadeModel(GL_SMOOTH);	// gouraud shading

//    puts(glGetString(GL_EXTENSIONS));

#ifdef USE_THREADS    
    assert(!sem_init(&diverg_mutex, 0, 1));
    assert(!sem_init(&x_mutex, 0, 1));

    pthread_attr_t attrs;
    pthread_attr_init(&attrs);
    pthread_attr_setdetachstate(&attrs, PTHREAD_CREATE_DETACHED);

    assert(!pthread_create(&req_thread, &attrs, &::processes_requests, (void *)this));

#endif
    
    connect_to_method(refresh, this, &fractal::queue_refresh);
    
    request(FRACT_RECOMPUTE_REQ);
    request(FRACT_REFRESH_REQ);
}


void fractal::queue_refresh() { 
#ifdef USE_THREADS    
    request(FRACT_REFRESH_REQ);
#else
    refresh_impl();
#endif
}

//int fuck_count = 0;

bool fractal::divergent(double s_val, double a_val, 
			double b_val, double c_val,
			eq_table *my_table, float * color) {

  //    printf("divergent - table = %p, table->q_index = %d\n", my_table, my_table->q_index);

    bool divergent;

    double vals[4];

    char key[sizeof(double)*4+1];

    HASH_TO_STRING_4(s_val,a_val,b_val,c_val,key);

    if(!diverg_cache->lookup(key, divergent)) {

	double initval = s_val*s_val+a_val*a_val+b_val*b_val+c_val*c_val;
	
	vals[0] = s_val;
	vals[1] = a_val;
	vals[2] = b_val;
	vals[3] = c_val;
	
	int iters;

	iters = (*params->function)(my_table, vals);
	double val = 
	    vals[0]*vals[0]+vals[1]*vals[1]+
	    vals[2]*vals[2]+vals[3]*vals[3];

// 	if(color)			// only set color for middle points
// 	    *color = (float)(initval/val);

	divergent = iters != max_iter;

// 	printf("entering %s, %d\n", buf, fuck_count++);
	diverg_cache->enter(key, false)->data = divergent;
    }

    //    printf("exitfff\n");
    return divergent;
}

void normalizeNormal(fcoord * p, void * f) {

#if 0

    for(DList<fpoly*> * l = p->polys ; l ; l = l->next) 
	if(!l->data) {
//	    printf("fucking shit !!!!!\n");
	    exit(-1);
	}
	    
#endif

#if 1
    if(!p->normalized) {
	
	p->norm[0] = p->norm[1] = p->norm[2] = 0;

// 	printf("point %p polys = %p:\n", p->polys);
	for(DList<fpoly*> *l = p->polys ; l ; l = l->next) {
#if 0
 	    printf("poly %p, %.15f %.15f %.15f\n", l->data, l->data->norm[0],
		   l->data->norm[1], l->data->norm[2]);
	    printf("vert %p, %.15f %.15f %.15f\n", l->data->vert[0], 
		   l->data->vert[0]->pos[0], l->data->vert[0]->pos[1],
		   l->data->vert[0]->pos[2]);
	    printf("vert %p, %.15f %.15f %.15f\n", l->data->vert[1], 
		   l->data->vert[1]->pos[0], l->data->vert[1]->pos[1], 
		   l->data->vert[1]->pos[2]);
	    printf("vert %p, %.15f %.15f %.15f\n\n", l->data->vert[2],
		   l->data->vert[2]->pos[0], l->data->vert[2]->pos[1],
		   l->data->vert[2]->pos[2]);
#endif
	    p->norm[0] += l->data->norm[0];
	    p->norm[1] += l->data->norm[1];
	    p->norm[2] += l->data->norm[2];
	}
///	printf("\n");
// 	printf("pre-norm = %.5f %.5f %.5f\n", p->norm[0], p->norm[1], p->norm[2]);

	float mag = sqrt(p->norm[0]*p->norm[0]+
			 p->norm[1]*p->norm[1]+
			 p->norm[2]*p->norm[2]);
	if(mag != 0.0) {
	    p->norm[0] /= mag;
	    p->norm[1] /= mag;
	    p->norm[2] /= mag;
	} else {
// 	    printf("fuck!!\n");
	    printf("%.15f %.15f %.15f\n", p->norm[0], p->norm[1], p->norm[2]);
	    printf("polys = %p\n", p->polys);
// 	    exit(-1);
	    p->norm[0] = 0;
	    p->norm[1] = 0;    
	    p->norm[2] = 0;
	}
// 	printf("post-norm = %.5f %.5f %.5f\n", p->norm[0], p->norm[1], p->norm[2]);
#if 0
	p->color[0] = fabs(p->norm[0]);
	p->color[1] = fabs(p->norm[1]);
	p->color[2] = fabs(p->norm[2]);
#else
	p->color[0] = (p->norm[0]+1)/2;
	p->color[1] = (p->norm[1]+1)/2;
	p->color[2] = (p->norm[2]+1)/2;
#endif
	p->normalized = true;
    }
#else

    fractal * fract = (fractal*)f;
    
				// from jules bloomenthal
    if(!p->normalized) {
	
	p->norm[0] = p->norm[1] = p->norm[2] = 0;
	int iters;
	double delta = 0.0025;

	double f = fract->table->iter(fract->w_val[1],
				      p->pos[0], p->pos[1], p->pos[2], 
				       iters);
	p->norm[0] = fract->table->iter(p->pos[0]+delta, p->pos[1], p->pos[2],
					fract->w_val[1], iters) - f;
	p->norm[1] = fract->table->iter(p->pos[0], p->pos[1]+delta, p->pos[2],
					fract->w_val[1], iters) - f;
	p->norm[2] = fract->table->iter(p->pos[0], p->pos[1], p->pos[2]+delta,
					fract->w_val[1], iters) - f;
	f = sqrt(p->norm[0]*p->norm[0] + p->norm[1]*p->norm[1] +
		 p->norm[2]*p->norm[2]);
	if (f != 0.0) {
	    p->norm[0] /= f;
	    p->norm[1] /= f;
	    p->norm[2] /= f;       
	}
    }
    
    p->normalized = true;
#endif    
}

void fractal::identifySurface(bool sizechange) {

    if(have_params)			// table deleted here
	delete params;
    have_params = true;
    if(tree)
	delete tree;

    params = new tree_params;

    printf("expr is %s\n", state->gui.expr_string);
    printf("init is %s\n", state->gui.init_expr_string);
    printf("maxiter is %.2f\n", state->gui.maxiter.gui_value());
    printf("maxiter is %d\n", (int)(state->gui.maxiter.gui_value()));
    table = new eq_table(state->gui.expr_string, state->gui.init_expr_string, 
			 (int)(state->gui.maxiter.gui_value()), max_val);
    max_iter = (int)(state->gui.maxiter.gui_value());
    params->function = table->compile();

    params->table = table;

    params->fract = this;

#if 0 
//USE_THREADS    
    int n = num_threads;
    pthread_t thread[n];

    if(state->gui.q_is_c_initially) 
	for(int i = 0 ; i < n ; i++) 
	    table[i]->set_q_to_c_initially();
    else
	for(int i = 0 ; i < n ; i++) 
	    table[i]->set_init_q_val(state->gui.initial_q_val);	
#else

//     if(state->gui.q_is_c_initially) 
// //	table->set_q_to_c_initially();
//     else 
// 	table->set_init_q_val(state->gui.initial_q_val);

#endif

    w_val[1] = 0;//state->gui.w_fix.value();

    params->min_level = (int)state->gui.init_tree_depth.value();
//     params->max_level = state->gui.tree_max_depth.value();
  
    diverg_cache->clear();

    params->points = new Hash<fcoord *>((params->min_level*
					 params->min_level*
					 params->min_level)*100+17,
					3*sizeof(double));
    
    tree = new octree(0, params, -2, -2,- 2, 4, 4, 4);
    
    tree->generate_polygons();
//     tree->print_polys();
    if(state->gui.gui)
	state->gui.gui->update_stats();
    
    params->points->foreach(normalizeNormal, this);

#if 0
    struct fcoord * p = params->first_pt;
    
    interleave = new float[(params->last_pt->index+1)*10];
    float * inter_ptr = interleave;

    while(p) {
	p->normalizeNormal();
//	p->multNormal(100);
//	p->dumpNormal();
//	p->dumpColor();
	//	printf("%d ", p->index);
//	p->dumpPoint();

	memcpy(inter_ptr, p, sizeof(float)*10);

	inter_ptr += 10;
	p = p->next;
    }

    glInterleavedArrays(GL_C4F_N3F_V3F, 0, interleave);

//     poly_count = params->last_poly->index+1;

    indices = new unsigned int[poly_count*3];
    unsigned int *index_ptr = indices;

    for(fpoly * poly = params->first_poly ; poly ; poly = poly->next) {
	for(int i = 0 ; i < 3 ; i++) {
	    index_ptr[i] = poly->vert[i];
// 	    printf("%.2f", ((f_hack*)interleave)[index_ptr[i]]);
// 	    printf(" (%d) ", index_ptr[i]);
	    if(index_ptr[i] > params->last_pt->index) {
		printf("index %d encountered, should be no more than %d\n",
		       index_ptr[i], params->last_pt->index);	
		exit(-1);
	    }		      
	}
	index_ptr += 3;
    }
#endif
    
//     delete params->first_pt;
//     params->first_pt = params->last_pt = NULL;

//     fpoly * f = params->first_poly;
//     fpoly * g;

//     while(f) {
// 	g = f;
// 	f = f->next;
// 	delete g;
//     }
    
//     delete params->first_poly;
//     params->first_poly = params->last_poly = NULL;

    printf("exit identifySurface");
    fflush(stdout);

}

void fractal::compute_projection() {

#define DIV 10

    if(!this || !state)
	return;

    glMatrixMode( GL_PROJECTION );
    glLoadIdentity();

    if(state->lw.view.perspective_on) {
      glFrustum(tree->min.x, tree->min.x+tree->span.x,
		tree->min.y, tree->min.y+tree->span.y,
		tree->span.z,
		tree->span.z*10);
    } else		     
      glOrtho(tree->min.x*1.5, (tree->min.x+tree->span.x)*1.5,
	      tree->min.y*1.5, (tree->min.y+tree->span.y)*1.5, 
	      tree->span.z, tree->span.z*10);
    
    gluLookAt(state->gui.pan.x.gui_value(), // from position
	      state->gui.pan.y.gui_value(),
	      state->gui.pan.z.gui_value()+tree->span.z*2,
	      state->gui.pan.x.gui_value(), // to position
	      state->gui.pan.y.gui_value(),
	      state->gui.pan.z.gui_value(),
	      0,1,0);		// up vector 


    double zoom_val = state->lw.view.get_zoom_val();
    
    glTranslatef(state->gui.pan.x.gui_value(),
		 state->gui.pan.y.gui_value(),
		 state->gui.pan.z.gui_value());

    glScalef(zoom_val, zoom_val, zoom_val);
    glRotatef(state->lw.view.rot.gety(), 1.0, 0.0, 0.0);
    glRotatef(state->lw.view.rot.getx(), 0.0, 1.0, 0.0);

    glTranslatef(-state->gui.pan.x.gui_value(),
		 -state->gui.pan.y.gui_value(),
		 -state->gui.pan.z.gui_value());

    glMatrixMode( GL_MODELVIEW );

}

#ifndef USE_THREADS

void fractal::request(short what) {
    switch(what) {
    case FRACT_REFRESH_REQ:
	refresh_impl();
	break;

    case FRACT_RECOMPUTE_SIZECHANGE_REQ:
	recompute(true);
	break;	    
	    
    case FRACT_RECOMPUTE_REQ:
	recompute(false);
	break;
    }
}

#endif // USE_THREADS


void fractal::refresh_impl()	// default impl for refresh signal
{
//    clock_t start = clock();

//    printf("refresh_init\n");

//    if(!state->fract)
//	return;

    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
    
//    printf("pre view\n");

    compute_projection();
    
//    printf("pre light\n");

    draw_light();
    
//    printf("post light\n");

    glMatrixMode(GL_MODELVIEW);	// draw fractal
    glLoadIdentity();
    glRotatef(-state->lw.model.rot.gety(), 1.0, 0.0, 0.0);
    glRotatef(-state->lw.model.rot.getx(), 0.0, 1.0, 0.0);

//     glTranslatef(((double)(state->gui.size.x.value()))/2,
//  		 ((double)(state->gui.size.y.value()))/2, 
//  		 ((double)(state->gui.size.z.value()))/2);
#if 0
#if 1
    glDrawElements(GL_TRIANGLES, poly_count*3, GL_UNSIGNED_INT, indices);
#else
    f_hack * h = (f_hack*)interleave;
    
    unsigned int * f = indices;
    
    glBegin(GL_TRIANGLES);
    
    while(f - indices < poly_count*3) {
      glColor4fv(h[*f].color);
      glNormal3fv(h[*f].norm);
      glVertex3fv(h[*f].pos);
	
//       printf("%.2f %.2f %.2f\n", h[*f].pos[0], h[*f].pos[1], h[*f].pos[2]);

      if((f-indices) % 3 == 2)
//	printf("\n");

      f++;
    }
    glEnd();
#endif
#else

    fpoly * polyp = params->first_poly;

    glBegin(GL_TRIANGLES);
    
    while(polyp) {

// 	glNormal3fv(polyp->norm);

	glColor4fv(polyp->vert[0]->color);
 	glNormal3fv(polyp->vert[0]->norm);
	glVertex3fv(polyp->vert[0]->pos);

	glColor4fv(polyp->vert[1]->color);
 	glNormal3fv(polyp->vert[1]->norm);
	glVertex3fv(polyp->vert[1]->pos);

	glColor4fv(polyp->vert[2]->color);
 	glNormal3fv(polyp->vert[2]->norm);
	glVertex3fv(polyp->vert[2]->pos);
    
	polyp = polyp->next;
    }

    glEnd();

    
    
#endif
//     glBegin(GL_TRIANGLES);
    
//     glColor4f(0.4, 0.2, 0.1, 0.5);
//     glNormal3f(1, 0, 0);
//     glVertex3f(0, 0, 0);
//     glVertex3f(0, 0, 1);
//     glVertex3f(0, 1, 1);
    
//     glEnd();

//     glBegin(GL_LINES);
    
//     glColor4f(1.0, 1.0, 1.0, 1.0);

//     glVertex3f(3,0,0);
//     glVertex3f(-3,0,0);

//     glVertex3f(0,3,0);
//     glVertex3f(0,-3,0);

//     glVertex3f(0,0,3);
//     glVertex3f(0,0,-3);
    
//     glEnd();


//    printf("refresh_post drawElements\n");

    if(state->gui.bounding_box_visible)
	drawBoundingBox();

    if(state->gui.zoom_box_visible)
	drawZoomBox();

//    printf("refresh : pre swapBuffers\n");
//    printf("drawable = %p\n", drawable);
    gdk_threads_enter();
    glXSwapBuffers(dpy,drawable);
    gdk_threads_leave();

//    printf("refresh_exit\n");

//    clock_t finish = clock();

//     printf("refresh took %.2f seconds CPU time\n", 
// 	   (double)(finish - start)/CLOCKS_PER_SEC);
}

void fractal::recompute(bool sizechange) {
    identifySurface(sizechange);
}

void fractal::drawZoomBox() {

    double min_x = tree->min.x;
    double min_y = tree->min.y;
    double min_z = tree->min.z;

    double span_x = tree->span.x;
    double span_y = tree->span.y;
    double span_z = tree->span.z;

    double x_mid = span_x/2+min_x;
    double y_mid = span_y/2+min_y;
    double z_mid = span_z/2+min_z;

    span_x *= state->gui.zoom.x.gui_value();
    span_y *= state->gui.zoom.y.gui_value();
    span_z *= state->gui.zoom.z.gui_value();
    
    span_x *= 0.5;
    span_y *= 0.5;
    span_z *= 0.5;  
    
    x_mid += state->gui.pan.x.gui_value();
    y_mid += state->gui.pan.y.gui_value();
    z_mid += state->gui.pan.z.gui_value();

    min_x = x_mid - span_x;
    min_y = y_mid - span_y;
    min_z = z_mid - span_z;
 
    double max_x = x_mid + span_x;
    double max_y = y_mid + span_y;
    double max_z = z_mid + span_z;
    
//     printf("check end\n");

    glBegin(GL_LINE_LOOP);
    
    glColor3f(1.0, 1.0, 1.0);
    
    glVertex3f(min_x, min_y, min_z);
    glVertex3f(min_x, max_y, min_z);
    glVertex3f(max_x, max_y, min_z);
    glVertex3f(max_x, min_y, min_z);

    glEnd();

    glBegin(GL_LINE_LOOP);

    glVertex3f(min_x, min_y, max_z);
    glVertex3f(min_x, max_y, max_z);
    glVertex3f(max_x, max_y, max_z);
    glVertex3f(max_x, min_y, max_z);

    glEnd();

    glBegin(GL_LINES);

    glVertex3f(min_x, min_y, max_z);
    glVertex3f(min_x, min_y, min_z);

    glVertex3f(min_x, max_y, max_z);
    glVertex3f(min_x, max_y, min_z);

    glVertex3f(max_x, min_y, max_z);
    glVertex3f(max_x, min_y, min_z);

    glVertex3f(max_x, max_y, max_z);
    glVertex3f(max_x, max_y, min_z);

    glEnd();

}

void fractal::drawBoundingBox() {
      
    float incr;

    glBegin(GL_LINES);

    glColor3f(1.0, 0.0, 0.0);

    for(float i = 0 ; i <= state->gui.bounding_res.value() ; i++) {
	
	incr = i/state->gui.bounding_res.value();

	glVertex3f(tree->min.x, 
		   tree->min.y, 
		   tree->min.z+incr*tree->span.z);
	glVertex3f(tree->min.x+tree->span.x,
		   tree->min.y, 
		   tree->min.z+incr*tree->span.z);

	glVertex3f(tree->min.x, 
		   tree->min.y+tree->span.y,
		   tree->min.z+incr*tree->span.z);
	glVertex3f(tree->min.x+tree->span.x, 
		   tree->min.y+tree->span.y, 
		   tree->min.z+incr*tree->span.z);

	glVertex3f(tree->min.x+tree->span.x,
		   tree->min.y+tree->span.y,
		   tree->min.z+incr*tree->span.z);
	glVertex3f(tree->min.x+tree->span.x,
		   tree->min.y,
		   tree->min.z+incr*tree->span.z);

	glVertex3f(tree->min.x,
		   tree->min.y+tree->span.y, 
		   tree->min.z+incr*tree->span.z);
	glVertex3f(tree->min.x, 
		   tree->min.y, 
		   tree->min.z+incr*tree->span.z);


	glVertex3f(tree->min.x+incr*tree->span.x,
		   tree->min.y, 
		   tree->min.z);
	glVertex3f(tree->min.x+incr*tree->span.x,
		   tree->min.y, 
		   tree->min.z+tree->span.z);

	glVertex3f(tree->min.x+incr*tree->span.x, 
		   tree->min.y+tree->span.y,
		   tree->min.z);
	glVertex3f(tree->min.x+incr*tree->span.x,
		   tree->min.y+tree->span.y,
		   tree->min.z+tree->span.z);

	glVertex3f(tree->min.x+incr*tree->span.x,
		   tree->min.y+tree->span.y, 
		   tree->min.z+tree->span.z);
	glVertex3f(tree->min.x+incr*tree->span.x,
		   tree->min.y,
		   tree->min.z+tree->span.z);

	glVertex3f(tree->min.x+incr*tree->span.x,
		   tree->min.y+tree->span.y,
		   tree->min.z);
	glVertex3f(tree->min.x+incr*tree->span.x,
		   tree->min.y, 
		   tree->min.z);


	glVertex3f(tree->min.x,
		   tree->min.y+incr*tree->span.y,
		   tree->min.z);
	glVertex3f(tree->min.x, 
		   tree->min.y+incr*tree->span.y,
		   tree->min.z+tree->span.z);

	glVertex3f(tree->min.x+tree->span.x,
		   tree->min.y+incr*tree->span.y, 
		   tree->min.z);
	glVertex3f(tree->min.x+tree->span.x,
		   tree->min.y+incr*tree->span.y, 
		   tree->min.z+tree->span.z);

	glVertex3f(tree->min.x, 
		   tree->min.y+incr*tree->span.y,
		   tree->min.z+tree->span.z);
	glVertex3f(tree->min.x+tree->span.x, 
		   tree->min.y+incr*tree->span.y,
		   tree->min.z+tree->span.z);

	glVertex3f(tree->min.x, 
		   tree->min.y+incr*tree->span.y,
		   tree->min.z);
	glVertex3f(tree->min.x+tree->span.x, 
		   tree->min.y+incr*tree->span.y, 
		   tree->min.z);
    }

    glEnd();
}


void fractal::draw_light() {

    if(this && state->lw.light.on) {
  
	glMatrixMode(GL_MODELVIEW);
	glPushMatrix();
	glLoadIdentity();
	
 	glEnable(GL_LIGHTING);
 	glEnable(GL_LIGHT0);

//	radius = fract->size.x;
	state->lw.light.pos[Z] = 0;

   	glRotatef(-state->lw.light.rot.gety(), 1.0, 0.0, 0.0);
   	glRotatef(-state->lw.light.rot.getx(), 0.0, 1.0, 0.0);
	glTranslatef( 0, 0, exp(state->lw.light.getRadius()));
	// 	float scale_val = state->lw.light.getRadius();
				// this one scales the light itself..
	// 	glScalef(scale_val, scale_val, scale_val);
				// next declare the light	
   	glMaterialfv(GL_FRONT, GL_AMBIENT, state->lw.light.mat_amb);
//   	glMaterialfv(GL_FRONT, GL_DIFFUSE, mat_dif);
//  	glMaterialfv(GL_FRONT, GL_SPECULAR, mat_specular);
//   	glMaterialfv(GL_FRONT, GL_SHININESS, mat_shininess);
   	glMaterialfv(GL_FRONT, GL_EMISSION, state->lw.light.mat_emission);
 
 	glLightfv( GL_LIGHT0, GL_POSITION,  state->lw.light.pos );
 	glLightfv( GL_LIGHT0, GL_SPOT_DIRECTION, state->lw.light.at);

 	glLightfv( GL_LIGHT0, GL_AMBIENT,  state->lw.light.ambient );
 	glLightfv( GL_LIGHT0, GL_SPECULAR, state->lw.light.specular );
 	glLightfv( GL_LIGHT0, GL_DIFFUSE,  state->lw.light.diffuse );

	float unit = (float)(tree->span.x)/12;

	glBegin(GL_TRIANGLE_FAN);

	glColor3f( 0.0, 0.0, 1.0);
	glVertex3f(0,0,unit);
 	glColor3f( 1.0, 1.0, 1.0 );
	glVertex3f( unit, unit, -unit );
	glVertex3f(-unit, unit, -unit );
	glVertex3f(-unit,-unit, -unit );
	glVertex3f( unit,-unit, -unit );
	glVertex3f( unit, unit, -unit );
	
	glEnd();
//  	glBegin( GL_LINE_LOOP );
//  	/* first draw bottom square and vertical lines */
//  	glVertex3f( -unit,-unit,-unit );
//  	glVertex3f( -unit, unit,-unit );
//  	glVertex3f( -unit,-unit,-unit );
//  	glVertex3f(  unit,-unit,-unit );
//  	glVertex3f(  unit, unit,-unit );
//  	glVertex3f(  unit,-unit,-unit );
//  	glVertex3f(  unit,-unit, unit );
//  	glVertex3f(  unit, unit, unit );
//  	glVertex3f(  unit,-unit, unit );
//  	glVertex3f( -unit,-unit, unit );
//  	glVertex3f( -unit, unit, unit );
//  	glVertex3f( -unit,-unit, unit );
//  	glEnd();

//  	glBegin( GL_LINE_LOOP );
//  	/* now draw top square of the light source */
//  	glVertex3f( -unit, unit,-unit );
//  	glVertex3f(  unit, unit,-unit );
//  	glVertex3f(  unit, unit, unit );
//  	glVertex3f( -unit, unit, unit );
//  	glEnd();


 	glPopMatrix();  

     } else {
	glDisable(GL_LIGHTING);
     }
}
