/*
 *  fractal.h
 * 
 *  Fractal Generator
 * 
 *  Copyright (c) 1998 Brian Martin
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef FRACTAL_H
#define FRACTAL_H

#include <GL/glx.h>
#include <gtk--.h>

#ifdef USE_THREADS
#include <pthread.h>
#include <semaphore.h>
#include "queue.h"
#endif // USE_THREADS

#include "fract_structs.h"
#include "octree.h"
#include "hash.h"
#include <libquat/expr_tree.h>
#include "light.h"
#include "glport.h"
#include "state.h"
#include <libquat/eq_table.h>
#include "fractal_state.h"
#include "octree.h"

#define DIVERGES_0 0
#define DIVERGES_1 1
#define DIVERGES_2 2
#define SURFACE    3

#define FRACT_RECOMPUTE_REQ 0
#define FRACT_REFRESH_REQ 1
#define FRACT_RECOMPUTE_SIZECHANGE_REQ 3

class fractal : public virtual Gtk_Signal_Base {
   
private:

public:

    class fractal_State *state;
    
    class octree * tree;

    double max_val;		// max escape value
    double max_iter;		// maxiumum iterations
    
    double w_val[3];		// values for w point
    double s_mod, a_mod, b_mod;	// increments;

    // polygons and points for the rendered model

    class tree_params *params;
    bool have_params;		/* this is a hack */
    
    int poly_count;
    float * interleave;		// interleaved array of packed vertex info
    unsigned int * indices;	// indexes into interleave specifying polygons

    class Hash<bool> * diverg_cache;
    
    class Fract_View *view;
    class Fract_Params *gui;

    Display *dpy;
    Window drawable;

    GLPort * glwin;
				/* maybe fix this at some point? */
/*     Fract_Progress progress; */

    Signal0 refresh;

    void free_position();

    eq_table * table;

#ifdef USE_THREADS
    queue requests;
    pthread_t req_thread;

    int num_threads;
    int currthread;

    int get_thread_num();	// called by divergence threads
    
    int currblock;
    sem_t diverg_mutex;
    sem_t x_mutex;		// for mutex on X stuff
    
    void change_num_threads(int newval) {
	if(newval > num_threads); // should probably do something here..
	
	num_threads = newval;
    }

    void request(short what) {
      // 	printf("queueing request %d\n", what);
 	requests.insert(what);
    }

    void processes_requests();		// executed by req_thread
#else
    void request(short what);

#endif // USE_THREADS

    void alloc();

    fractal(class fractal_State *);

				// finds surface based upon divergence info
    void identifySurface(bool sizechange); 
    void computeSurface();	// generates polygons from surface

    // determines if point has any divergent neibhors
    bool divergentNeibhor(int i, int j, int k); 

    // determines convergance (divergance) of point
    bool divergent(double, double, double, double, eq_table *, float*);	
#ifdef USE_THREADS
    void test_divergence();	// for threads
    int get_next_divergence_block(); 
#endif // USE_THREADS

    // lists neibhors of given point

    void draw();		// draws model from polygons
    void drawBox(int i, int j, int k, float); // low level drawing primitives
    void drawCube(int i, int j, int k, float color);	
    int addPoint(int x_pos, int y_pos, int j_pos,
		 int x_norm, int y_norm, int j_norm,
		 float color);
    void addPoint(int x_pos, int y_pos, int j_pos, float color);
    void addPoly(int, int, int, int);

    void readFrom(FILE * file);	// these don't work right now..
    void writeTo(FILE * file);
    
				// use given function for fractal generation
    void set_expr(const char * expr); 
    void init();		// sets up default paramaters
    void gl_init();		// sets up gl
    void recompute(bool);	// calls {compute,identify}Surface
    void refresh_impl();	// redraws fractal
    void queue_refresh();	// callback for refresh signal
    void drawBoundingBox();	
    void drawZoomBox();
    void compute_projection();
    void draw_light();
};

#endif




