/*
 *  glport.cc
 * 
 *  Fractal Generator
 * 
 *  Copyright (c) 1998 Brian Martin
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*
 * glgtk.c
 *
 * Release: 971104
 *
 * Displays a triangle with Open/GL | Mesa
 *
 * Copyright (c) 1997 by Ivo Clarysse, <soggie@riv.be>
 *
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include <gtk--.h>

#include <gtk/gtk.h>
#include <gdk/gdk.h>
#include <gdk/gdkx.h>

extern "C" {
#include <GL/gl.h>
#include <GL/glx.h>
#include <GL/glu.h>
}

#include "glport.h"

GLPort::GLPort(int x, int y, char * title) {

    glarea = new Gtk_DrawingArea;
    if(!glarea) {
	printf("glarea == NULL\n");
	exit(-1);	
    }
    glarea->size(x, y);
    glarea->show();
    add(glarea);

    set_title(title);

    connect_to_method(glarea->expose_event, this, &GLPort::handle_expose);

    glarea->set_events(GDK_EXPOSURE_MASK);
}

void GLPort::show_impl() {

//     GtkWindowClass *sig=static_cast<GtkWindowClass *>(get_parent_class());
// //    if (!sig->show) return;
//     sig->show(gtkobj());
    Gtk_Widget::show_impl();
    
    connect_to_method(glarea->configure_event, this, &GLPort::handle_configure);

				// gl stuffs
    gint TimerTag;

    XVisualInfo *vi;
    int dbuf[] = {GLX_RGBA,GLX_RED_SIZE,1,GLX_GREEN_SIZE,1,
		  GLX_BLUE_SIZE,1,GLX_DOUBLEBUFFER,GLX_DEPTH_SIZE,1,None};

    /* Choose a visual that OpenGL is happy with.. */
    dpy=GDK_DISPLAY();
    vi=glXChooseVisual(dpy,DefaultScreen(dpy),dbuf);

    /* Initialize OpenGL */

    if (vi==NULL) { 
	fprintf(stderr,"ERROR: Could not create visual\n"); 
	exit(-1); 
    }

    printf("Selected visual = 0x%x\n",(unsigned int)(vi->visualid));
    glx_context = glXCreateContext(dpy,vi,None,GL_TRUE);

    if (glx_context==NULL) { 
	fprintf(stderr,"ERROR: Could not create context\n"); 
	exit(-1); 
    }

    /* Initialize OpenGL context for rendering */
    printf("glXMakeCurrent\n");
    glXMakeCurrent(dpy, 
		   GDK_WINDOW_XWINDOW(GTK_WIDGET(glarea->gtkobj())->window),
		   glx_context);

    glShadeModel(GL_FLAT);
    glClearColor(0.0,0.0,0.0,0.0);
    glColor3f(1.0, 1.0, 1.0);
    glLineWidth(1.0);


    glMatrixMode(GL_PROJECTION);   
    glLoadIdentity();
    glFrustum(-1.0, 1.0, -1.0, 1.0, 1.5, 20.0);
    glViewport(0, 0, width(), height());
}

Window GLPort::getWin() {
    return(GDK_WINDOW_XWINDOW(GTK_WIDGET(glarea->gtkobj())->window));
}

/*
 * Callback for GTK events in glarea 
 *
 */
gint GLPort::handle_configure(GdkEventConfigure * event) {

    printf("entry: GDK_CONFIGURE\n");
    
    /* Resize OpenGL context in glarea */
    glMatrixMode(GL_PROJECTION);   
    glLoadIdentity();             
    glFrustum(-1.0, 1.0, -1.0, 1.0, 1.5, 20.0);
				// need to use these for aspect ratio in
				// glFrustum
    glViewport(0, 0, event->width, event->height);
    glMatrixMode(GL_MODELVIEW);	

    printf("exit: GDK_CONFIGURE\n");
    return 0;
}

gint GLPort::handle_expose(GdkEventExpose *event)
{

				// should put something here

}

