/*
 *  hash.h
 * 
 *  Fractal Generator
 * 
 *  Copyright (c) 1998 Brian Martin
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef HASH_H
#define HASH_H

#undef DEBUG

#include <stdio.h>
#include <semaphore.h>

#define HASH_TO_STRING_3(a,b,c,f) *((double*)f)=a;((double*)f)[1]=b;((double*)f)[2]=c;

#define HASH_TO_STRING_4(a,b,c,f,g) *((double*)g)=a;((double*)g)[1]=b;((double*)g)[2]=c;((double*)g)[3]=f;

#define HASH_TO_STRING_4V(a,b) memcpy(b,a,sizeof(double)*4)

#define HASH_TO_STRING_3V(a,b) memcpy(b,a,sizeof(double)*3)

template<class T> class Hash_node {

public:

    char * string;
    T data;
    struct Hash_node<T> * next;
/*     struct Hash_node<T> * prev; */

    Hash_node(): string(0), next(0), data(0) {}

    ~Hash_node() {
  	if(string) 
  	    delete [] string; 
/*  	if(next)  */
/*  	    delete(next);  */
    }
};

template<class T> class Hash {
    
private:

    Hash_node<T> ** table;
    int size;
    int len;
#ifdef USE_THREADS
    sem_t mutex;
#endif

public:

    int Hashpwj(char * data);

    bool lookup( char * string, T & data );
    Hash_node<T> * enter( char * string, bool lookup );
    bool remove(char * string);
    void clear();
    void foreach (void (*f)(T));
    void foreach (void (*f)(T,void*),void *);
//    void foreach (void (T::*method)(char*), char * arg);

    equal(char * a, char * b) {	/* assuming length % 4 == 0 */
	bool equal = true;
/* 	output(a);  */
/*  	output(b);  */
	for(int i = 0 ; equal && i < len/4 ; i++) 
	    equal = equal && !(((int*)a)[i]^((int*)b)[i]);
/*  	printf("are %d\n", equal); */
	return equal;
    }
    
    output(char * string) {
	for(int i = 0 ; i < len ; i++) 
	    printf("%x", string[i]);
/* 	    for(int j = 0 ; j < 8 ; j++)  */
/* 		if(string[i]&1<<j) */
/* 		    printf("1"); */
/* 		else */
/* 		    printf("0"); */
/* 	for(int i = 0 ; i < len/4 ; i++) */
	printf(" %.5f %.5f %.5f", ((double*)string)[0], ((double*)string)[1],
	       ((double*)string)[2]);
	printf("\n");
    }

    Hash(int _size,int _len): size(_size), len(_len) {
	table = new Hash_node<T> * [size];
	for(int i = 0 ; i < size ; i++)
	    table[i] = 0;
#ifdef USE_THREADS
	sem_init(&mutex, 0, 1);
#endif
    }

    ~Hash() {
	for(int i = 0 ; i < size ; i++) {
	    Hash_node<T> * n = table[i];
	    while(n) {
		Hash_node<T> * old = n;
		n = n->next;
 		delete old; 
	    }
	}
    }

};

template <class T>
int Hash<T>::Hashpwj(char * s) {
    char * p;
    unsigned h = 0, g;
    int i = 0;

    for ( p = s ; ++i <= len ; p++ ) {
	h = ( h << 4 ) + *p;
	if (g = h & 0xf0000000) {
	    h = h ^ ( g >> 24 );
	    h = h ^ g;
	}
    }
    return h % size;
}
/*
template <class T>
void Hash<T>::foreach (void (T::*f)(char*), char * arg) {

    Hash_node<T> * ptr;

    for(int i = 0 ; i < size ; i++)
	for(ptr = table[i] ; ptr ; ptr = ptr->next)
	    ((ptr->data)->*f)(arg);
}*/


template <class T>
void Hash<T>::foreach( void (*f)(T) ) {
    
    Hash_node<T> * ptr;

    for(int i = 0 ; i < size ; i++)
	for(ptr = table[i] ; ptr ; ptr = ptr->next)
	    (*f)(ptr->data);
}

template <class T>
void Hash<T>::foreach( void (*f)(T,void*), void * a1 ) {
    
    Hash_node<T> * ptr;

    for(int i = 0 ; i < size ; i++)
	for(ptr = table[i] ; ptr ; ptr = ptr->next)
	    (*f)(ptr->data,a1);
}

template <class T>
bool Hash<T>::lookup( char * string, T & data ) {

    int hashval = Hashpwj(string);
    bool found = false;

#ifdef USE_THREADS
    sem_wait(&mutex);
#endif
    
    Hash_node<T> * list = table[hashval];
    Hash_node<T> * new_node;

    while ( list != NULL ) {	// first look for string in Hash
	if (equal(string, list->string)) {
	    found = true;
	    data = list->data;
	    break;
	}
	list = list->next;
    }

#ifdef USE_THREADS
    sem_post(&mutex);
#endif

    return found;
}


template <class E> 
bool Hash<E>::remove(char * key) {

    int hashval = Hashpwj(key);

#ifdef DEBUG
    printf("remove: hashval is %d\n", hashval);
#endif
    
    bool found = false;

#ifdef USE_THREADS
    sem_wait(&mutex);
#endif

    Hash_node<E> * list = table[hashval];

    if(list != NULL) {
#ifdef DEBUG
 	output(list->string); 
	output(key); 
 	printf("\n"); 
#endif
	if(equal(key, list->string)) {
// 	    printf("removing:\n");
	    if(list->next) {
		Hash_node<E>*unfortunate_one = list;
		table[hashval] = list->next;
 		delete unfortunate_one; 
	    } else {
 		delete list; 
		table[hashval] = NULL;
	    }
	    found = true;
        } else {
            while( list->next != NULL ) {	       
#ifdef DEBUG
    		output(list->next->string);   
    		output(key);   
#endif
                if(equal(key, list->next->string)) {
                    found = true;
                    break;
                }
		list = list->next;
            }
	    if(found) {
// 		printf("removing:\n"); 
		Hash_node<E> * exitcutee = list->next;
		list->next = list->next->next;
 		delete exitcutee; 
	    } 
        }
    }

#ifdef USE_THREADS
    sem_post(&mutex);
#endif

#if 0
    char * fe = 0;
    if(!found)
	*fe = 0;
#endif

    return found;
}

template <class T> 
Hash_node<T> * Hash<T>::enter( char * string, bool lookup) {
    
    int hashval = Hashpwj(string);


    bool found = false;

#ifdef DEBUG
    if (len == sizeof(double)*3) {
 	printf("hashval = %d table[hashval] = %p\n", hashval, table[hashval]); 
 	printf("entering ");
 	output(string);
    }
#endif

#ifdef USE_THREADS
    sem_wait(&mutex);
#endif

    Hash_node<T> * list = table[hashval];
    Hash_node<T> * new_node;

    if(lookup) 
	while ( list != NULL ) {	// first look for string in Hash
	    if (equal(string, list->string)) {
		found = true;
		new_node = list;
		break;
	    }
	    list = list->next;
	}

    if(!found) {
	new_node = new Hash_node<T>;
	new_node->string = new char[len];
	memcpy(new_node->string,string,len);
	new_node->next = table[hashval];
	table[hashval] = new_node;
    }

/*     table[hashval]->dump(); */
/*     printf("\n"); */

#ifdef USE_THREADS
    sem_post(&mutex);
#endif

    return new_node;
}

template <class T> void Hash<T>::clear() {
    
    Hash_node<T> * list;
    Hash_node<T> * next;

    for(int i = 0 ; i < size ; i++) {
	
	list = table[i];
	while(list != NULL) {
	    next = list->next;
 	    delete list;
	    list = next;
	}
	table[i] = NULL;
    }
}



#endif // HASH_H
