#include "light_state.h"
#include "math.h"

float Light_State::getRadius() { 
    if(light)
	return exp(light->adj->get_value());
    else
	return radius;
}

void Light_State::save(FILE * fp) {
	    
    fprintf(fp, "%.5f %.5f %.5f %d\n", radius, min_rad, max_rad, on);
    rot.save(fp);

    save_vect(mat_amb, fp);
    save_vect(mat_dif, fp);
    save_vect(mat_specular, fp);
    save_vect(mat_emission, fp);
    save_vect(mat_shininess, fp);
    save_vect(db_ambient, fp);
    save_vect(db_diffuse, fp);
    save_vect(db_specular, fp);
    save_vect(db_at, fp);
    save_vect(ambient, fp);
    save_vect(diffuse, fp);
    save_vect(specular, fp);
    save_vect(at, fp);
    save_vect(pos, fp);	    

}

void Light_State::load(FILE * fp) {
	    
    fscanf(fp, " %f  %f  %f %d ", &radius, &min_rad, &max_rad, &on);
    
    rot.load(fp);

    load_vect(mat_amb, fp);
    load_vect(mat_dif, fp);
    load_vect(mat_specular, fp);
    load_vect(mat_emission, fp);
    load_vect(mat_shininess, fp);
    load_vect(db_ambient, fp);
    load_vect(db_diffuse, fp);
    load_vect(db_specular, fp);
    load_vect(db_at, fp);
    load_vect(ambient, fp);
    load_vect(diffuse, fp);
    load_vect(specular, fp);
    load_vect(at, fp);
    load_vect(pos, fp);	    
}
// operator != (State & other) {
//     return(light != other.light ||
// 	   changed != other.changed ||
// 	   radius != other.radius ||
// 	   on != other.on);
// }
void Light_State::save_vect(GLfloat * vect, FILE * fp) {
    for(int i = 0 ; i < 4 ; i++) 
	fprintf(fp, "%.5f ", vect[i]);
    fprintf(fp, "\n");
}
void Light_State::load_vect(GLfloat * vect, FILE * fp) {

    fscanf(fp, " %f %f %f %f ", vect, vect+1, vect+2, vect+3);
}
void Light_State::set_default() { 
    rot.xpos = 0;
    rot.ypos = 0; 
    radius = 10.0;
    min_rad = 0.01;
    max_rad = 100;
    on = false;
	    
    mat_amb[X] = 0.2;
    mat_amb[Y] = 0.3;
    mat_amb[Z] = 0.3;
    mat_amb[W] = 0.3;

    mat_dif[X] = 0.3;
    mat_dif[Y] = 0.3;
    mat_dif[Z] = 0.3;
    mat_dif[W] = 0.3;
    
    mat_specular[X] = 0.2;
    mat_specular[Y] = 0.2;
    mat_specular[Z] = 0.2;
    mat_specular[W] = 0.2;

    mat_emission[X] = 0.2;
    mat_emission[Y] = 0.2;
    mat_emission[Z] = 0.2;
    mat_emission[W] = 0.2;

    *mat_shininess = 3.0;

    pos[X] = 0.0; // x 
    pos[Y] = 0.0; // y 
    pos[Z] = radius; // z 
    pos[W] = 1.0; // homogeneous 

    at[X] = 0.0;
    at[Y] = 0.0;
    at[Z] = 0.0;
    at[W] = 1.0;

    ambient[X] = 0.2;
    ambient[Y] = 0.2;
    ambient[Z] = 0.2;
    ambient[W] = 1.0;

    diffuse[X] = 0.3;
    diffuse[Y] = 0.3;
    diffuse[Z] = 0.3;
    diffuse[W] = 0.3;

    specular[X] = 0.5;
    specular[Y] = 0.5;
    specular[Z] = 0.5;
    specular[W] = 0.9;    

}
Light * Light_State::create_gui() { return light = new Light(this); }
//	void Light_State::show_dialog() { light = new Light(this); }
void Light_State::apply() { rot.apply(); }
void Light_State::grab() { rot.grab(); }
