#ifndef OCTREE_H
#define OCTREE_H

#include "hash.h"
#include "fractal.h"
#include <libquat/eq_table.h>
#include "list.h"

#ifndef X
#define X 0
#endif
#ifndef Y
#define Y 1
#endif
#ifndef Z
#define Z 2
#endif

class coord {

public:

    double x;
    double y;
    double z;
};

class tree_params {
    
public:

    int min_level, max_level;
    class Hash<class fcoord *> * points;
    class fractal * fract;
    int last_point;
    eq_table *table;
    libquat_iter_func function;
    class fpoly * first_poly;

    int poly_count();

    tree_params();
    ~tree_params();
};

class octree {

private: 
    

    List<fpoly*> *polys;

    class octree * child[8];
    int level;
				// does binary search for point
				// (assmuing that coords have differing 
				// divergence info)
    coord search_for_point(coord, coord, int); 
    void alloc_children();
    bool not_void(int);
    inline void remove_polys();    

public:
    tree_params * params;

    int count();
    int max_depth();
    int min_depth();
    
    coord min;
    coord span;
    
    void print_polys();

    octree(int level, tree_params * _params,
	   double min_x, double min_y, double min_z,
	   double x_span, double y_span, double z_span);

    ~octree();

    void generate_polygons();
    void subdivide(int weight, double x_min, double y_min, double z_min,
		   double x_max, double y_max, double z_max);
    void make_polygons(int corner[4], coord coords[9], bool diverg[9]);
    void add_poly(coord x, float c1,
		  coord y, float c2,
		  coord z, float c3);
    fcoord * add_point(double x, double y, double z, float c);
};


#endif
