/*
 *  queue.h
 * 
 *  Fractal Generator
 * 
 *  Copyright (c) 1998 Brian Martin
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef QUEUE_H
#define QUEUE_H

#include <semaphore.h>


/*
  this is a synchronized queue implemented with an array
  
  remove() blocks until the queue is non empty
  insert() signals wait

  it is assumed that only one thread at a time will insert, and only one will 
  remove.  (no mutex) (maybe there should be)
 */
class queue {

private:
  
    char * Q;
    char * front;
    char * back;
    int size;
    sem_t empty;
    sem_t mutex;

public:

    queue(int num_elems) {
	Q = new char[num_elems];
	front = Q;
	back = Q;
	sem_init(&empty, 0, 0);
	sem_init(&mutex, 0, 1);
    }

    void insert(char what) {
	sem_wait(&mutex);
	*back = what;
	if(back - Q == size - 1) 
	    back = Q;
	else
	    back++;
	sem_post(&mutex);
	sem_post(&empty);
    }
    
    char remove() {
	sem_wait(&empty);
	sem_wait(&mutex);
	char ans = *front;
	if(front - Q == size -1)
	    front = Q;
	else
	    front++;
	sem_post(&mutex);
	return ans;
    }
    
    void clear_pending(char which) { 
	sem_wait(&mutex);
	while(*front == which && !sem_trywait(&empty)) {
	  //	    printf("clearing %d\n", which);
	    if(front - Q == size - 1) 
		front = Q;
	    else 
		front++;
	}
	sem_post(&mutex);
    }

    bool isEmpty() {
	return(front==back);
    }
};





#endif
