/*
 *  radial_positioner.cc
 * 
 *  Fractal Generator
 * 
 *  Copyright (c) 1998 Brian Martin
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "radial_positioner.h"
#include "stdio.h"

Gtk_RadialPositioner::Gtk_RadialPositioner() {

    /*
      position 0,0 is dead center
      y increases upward, x to the right
      */
      
#ifdef USE_THREADS
    sem_init(&mutex, 0, 1);
#endif
    xrot = yrot = xdisp = ydisp = 0;
    reversed = prev_in = drag = false;
    connect_to_method(event, this, &Gtk_RadialPositioner::handle_events);
}

Gtk_RadialPositioner::Gtk_RadialPositioner(int width, int heigth) {

    /*
      position 0,0 is dead center
      y increases upward, x to the right
      */
      
#ifdef USE_THREADS
    sem_init(&mutex, 0, 1);
#endif
    xrot = yrot = xdisp = ydisp = 0;
    reversed = prev_in = drag = false;
    size(width, heigth);
    connect_to_method(event, this, &Gtk_RadialPositioner::handle_events);
}

double Gtk_RadialPositioner::getx() { 
#ifdef USE_THREADS
    sem_wait(&mutex);
#endif
    double ret_val = reversed ? xrot : xrot;
#ifdef USE_THREADS
    sem_post(&mutex);
#endif
    return(ret_val); 
}

double Gtk_RadialPositioner::gety() { 
#ifdef USE_THREADS
    sem_wait(&mutex);
#endif
    double ret_val = reversed ? yrot : yrot;
#ifdef USE_THREADS
    sem_post(&mutex);
#endif
    return(ret_val);
}


void Gtk_RadialPositioner::draw_frame() {

    gint w, h;
    

#define FF 5
#define HH 4

    w = GTK_WIDGET(gtkobject)->allocation.width;
    h = GTK_WIDGET(gtkobject)->allocation.height;

    gdk_draw_arc (GTK_WIDGET(gtkobject)->window,
		  GTK_WIDGET(gtkobject)->style->black_gc,
		  FALSE, 1, 1, w - FF, h - FF, 30 * 64, 180 * 64);

    gdk_draw_arc (GTK_WIDGET(gtkobject)->window,
		  GTK_WIDGET(gtkobject)->style->mid_gc[GTK_STATE_NORMAL],
		  FALSE, 0, 0, w-HH, h-HH, 30 * 64, 180 * 64);

    gdk_draw_arc (GTK_WIDGET(gtkobject)->window, 
		  GTK_WIDGET(gtkobject)->style->bg_gc[GTK_STATE_NORMAL],
		  FALSE, 1, 1, w - FF, h - FF, 210 * 64, 180 * 64);

    gdk_draw_arc (GTK_WIDGET(gtkobject)->window,
		  GTK_WIDGET(gtkobject)->style->light_gc[GTK_STATE_NORMAL],
		  FALSE, 0, 0, w-HH, h-HH, 210 * 64, 180 * 64);
}

void Gtk_RadialPositioner::draw_position() {

    gint w, h;
    
    w = GTK_WIDGET(gtkobject)->allocation.width;
    h = GTK_WIDGET(gtkobject)->allocation.height;

    int top, height, side, width;
    
    height = sqrt(ydisp*ydisp/(1-(xdisp*xdisp)/(90*90)))*h/90;
    top = 0.5*(h-height);

    width = sqrt(xdisp*xdisp/(1-(ydisp*ydisp)/(90*90)))*w/90;
    side = 0.5*(w-width);

				// fill background
    gdk_draw_arc (GTK_WIDGET(gtkobject)->window,
		  GTK_WIDGET(gtkobject)->style->light_gc[GTK_STATE_NORMAL],
		  TRUE, 2, 2, w-HH, h-HH, 0, 360 << 6);

    gdk_draw_point(GTK_WIDGET(gtkobject)->window,
		   GTK_WIDGET(gtkobject)->style->black_gc,
		   w/2, h/2);

    if(reversed) {
	gdk_draw_arc(GTK_WIDGET(gtkobject)->window, 
		     GTK_WIDGET(gtkobject)->style->mid_gc[GTK_STATE_NORMAL],
		     FALSE, 2, top, w-FF, height, ydisp>0 ? 0 : 180<<6, 180<<6);
 
	gdk_draw_arc(GTK_WIDGET(gtkobject)->window, 
		     GTK_WIDGET(gtkobject)->style->mid_gc[GTK_STATE_NORMAL],
		     FALSE, side, 2, width, h-FF, (xdisp>0 ? 90:270)<<6, 180<<6);

	gdk_draw_arc(GTK_WIDGET(gtkobject)->window, 
		     GTK_WIDGET(gtkobject)->style->black_gc,
		     FALSE, 2, top, w-FF, height, ydisp>0 ? 180<<6 : 0, 180<<6);
 
	gdk_draw_arc(GTK_WIDGET(gtkobject)->window, 
		     GTK_WIDGET(gtkobject)->style->black_gc,
		     FALSE, side, 2, width, h-FF, (xdisp>0 ? 270:90)<<6, 180<<6);
    } else {
	gdk_draw_arc(GTK_WIDGET(gtkobject)->window, 
		     GTK_WIDGET(gtkobject)->style->mid_gc[GTK_STATE_NORMAL],
		     FALSE, 2, top, w-FF, height, ydisp>0 ? 180<<6 : 0, 180<<6);
 
	gdk_draw_arc(GTK_WIDGET(gtkobject)->window, 
		     GTK_WIDGET(gtkobject)->style->mid_gc[GTK_STATE_NORMAL],
		     FALSE, side, 2, width, h-FF, (xdisp>0 ? 270:90)<<6, 180<<6);
 
	gdk_draw_arc(GTK_WIDGET(gtkobject)->window, 
		     GTK_WIDGET(gtkobject)->style->black_gc,
		     FALSE, 2, top, w-FF, height, ydisp>0 ? 0 : 180<<6, 180<<6);
 
	gdk_draw_arc(GTK_WIDGET(gtkobject)->window, 
		     GTK_WIDGET(gtkobject)->style->black_gc,
		     FALSE, side, 2, width, h-FF, (xdisp>0 ? 90:270)<<6, 180<<6);
    }
}

void Gtk_RadialPositioner::fuck(double a, double b) {

    printf("%lf %lf\n", a, b);
}


void Gtk_RadialPositioner::set(double _xrot, double _yrot) {
 
    xrot = _xrot;
    yrot = _yrot;

    if(yrot*yrot+xrot*xrot > 90*90) { // radius is past edge of mask
	if(ydisp > 0)
	    ydisp = yrot-2*fabs(xrot)*(1-90/sqrt(yrot*yrot+xrot*xrot));
	else
	    ydisp = yrot+2*fabs(xrot)*(1-90/sqrt(yrot*yrot+xrot*xrot));
	if(xdisp > 0)
	    xdisp = xrot-2*fabs(yrot)*(1-90/sqrt(yrot*yrot+xrot*xrot));
	else
	    xdisp = xrot+2*fabs(yrot)*(1-90/sqrt(yrot*yrot+xrot*xrot));
	if(prev_in && drag)
	    reversed = !reversed;
	prev_in = false;
    } else {
	if(!prev_in && drag) 
  	    reversed = !reversed;
	prev_in = true;
	ydisp = yrot;
	xdisp = xrot;
    }
    //    printf("yrot = %lf, xrot = %lf, ydisp = %lf xdisp = %lf\n", yrot, xrot, ydisp, xdisp);
#ifdef USE_THREADS
//    sem_post(&mutex);
#endif
}

gint Gtk_RadialPositioner::handle_events(GdkEvent * event) {

    switch(event->type) {

    case GDK_EXPOSE:
	draw_frame();
	draw_position();	
	break;

    case GDK_BUTTON_PRESS:
	set(90 - event->motion.x*180/GTK_WIDGET(gtkobject)->allocation.width,
	    90 - event->motion.y*180/GTK_WIDGET(gtkobject)->allocation.width);
	draw_position();
	changed();
	break;

    case GDK_BUTTON_RELEASE:
	drag = false;
	break;

    case GDK_MOTION_NOTIFY:
	drag = true;
	set(90 - event->motion.x*180/GTK_WIDGET(gtkobject)->allocation.width,
	    90 - event->motion.y*180/GTK_WIDGET(gtkobject)->allocation.width);
       	draw_position();
	changed();
	break;

    }


}

