/*
 * glgtk.c
 *
 * Release: 971104
 *
 * Displays a triangle with Open/GL | Mesa
 *
 * Copyright (c) 1997 by Ivo Clarysse, <soggie@riv.be>
 *
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include <gtk--.h>

#include <gtk/gtk.h>
#include <gdk/gdk.h>
#include <gdk/gdkx.h>

extern "C" {
#include <GL/gl.h>
#include <GL/glx.h>
#include <GL/glu.h>
}

int redraw_needed=TRUE;

class Gtk_GLPort : public Gtk_Window {
	
private:

    GLXContext glx_context;
    Display *dpy;
    Gtk_DrawingArea * glarea;
    handle_configure(GdkEventConfigure *);
    handle_expose(GdkEventExpose *);
    void show_impl();

public:

    Gtk_GLPort(int x, int y);
    GdkWindow * getWindow();
    gint redraw(GdkEventExpose*);
};

Gtk_GLPort *glp;

GLfloat cam_xspin=0.0;
GLfloat cam_yspin=0.0;
GLfloat cam_zspin=0.0;
GLfloat cam_xpos=0.0;
GLfloat cam_ypos=0.0;
GLfloat cam_zpos=-4.0;

gint timer_callback(gpointer data)
{
    cam_zspin+=4;
    cam_xspin+=1;
    redraw_needed=TRUE;
    glp->redraw(NULL);  /* Redraw object if needed */
    return(TRUE);
}


void gui_init(GdkWindow * win)
{

}


/* Define model in displaylist */
void            object_define()
{
    glNewList(1, GL_COMPILE_AND_EXECUTE);
    glClear(GL_COLOR_BUFFER_BIT);

    glBegin(GL_LINE_LOOP);
    glVertex3f(-1, 0, 0);
    glVertex3f( 0, -1, 0);
    glVertex3f( 1, -1, 0);
    glEnd();
  
    glEndList();
}

Gtk_GLPort::Gtk_GLPort(int x, int y) {

    glarea = new Gtk_DrawingArea;
    if(!glarea) {
	printf("fuck:  glarea == NULL\n");
	exit(-1);	
    }
    glarea->size(x, y);
    glarea->show();
    add(glarea);

    connect(glarea->expose_event, this, &Gtk_GLPort::handle_expose);

    glarea->set_events(GDK_EXPOSURE_MASK);
}

void Gtk_GLPort::show_impl() {

    c_signals_Widget *sig=(c_signals_Widget *)internal_getsignalbase();
    if (!sig->show) return;
    sig->show((GtkWidget*)gtkobject);
    
    connect(glarea->configure_event, this, &Gtk_GLPort::handle_configure);

				// gl stuffs
    gint TimerTag;

    XVisualInfo *vi;
    int dbuf[] = {GLX_RGBA,GLX_RED_SIZE,1,GLX_GREEN_SIZE,1,
		  GLX_BLUE_SIZE,1,GLX_DOUBLEBUFFER,None};

    /* Choose a visual that OpenGL is happy with.. */
    dpy=GDK_DISPLAY();
    vi=glXChooseVisual(dpy,DefaultScreen(dpy),dbuf);

    printf("check\n");

    /* Initialize OpenGL */

    if (vi==NULL) { 
	fprintf(stderr,"ERROR: Could not create visual\n"); 
	exit(-1); 
    }

    printf("Selected visual = 0x%x\n",(unsigned int)(vi->visualid));
    glx_context = glXCreateContext(dpy,vi,None,GL_TRUE);

    if (glx_context==NULL) { 
	fprintf(stderr,"ERROR: Could not create context\n"); 
	exit(-1); 
    }

    /* Initialize OpenGL context for rendering */
    printf("glXMakeCurrent\n");
    glXMakeCurrent(dpy, 
		   GDK_WINDOW_XWINDOW(GTK_WIDGET(glarea->gtkobject)->window),
		   glx_context);

    glShadeModel(GL_FLAT);
    glClearColor(0.0,0.0,0.0,0.0);
    glColor3f(1.0, 1.0, 1.0);
    glLineWidth(1.0);

    /* Attach callback to timer */
    TimerTag=gtk_timeout_add(50,timer_callback,NULL);

    set_title("Gtk-- GL test");        
    redraw_needed=TRUE;

    glMatrixMode(GL_PROJECTION);   
    glLoadIdentity();             
    glFrustum(-1.0, 1.0, -1.0, 1.0, 1.5, 20.0);
    glViewport(0, 0, width(), height()); 

}

GdkWindow * Gtk_GLPort::getWindow() {
    return(GTK_WIDGET(glarea->gtkobject)->window);
//    return(NULL);
}

gint Gtk_GLPort::redraw(GdkEventExpose*) {

    static int list_inited=FALSE;

    if (!redraw_needed) return 0;

    glShadeModel(GL_FLAT);
    glClearColor(0.0,0.0,0.0,0.0);
    glColor3f(1.0, 1.0, 1.0);
    glLineWidth(1.0);

    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();

    glTranslatef(cam_xpos, cam_ypos, cam_zpos);
    glRotatef(cam_xspin, 1.0, 0.0, 0.0);        /* rotate modelmatrix */
    glRotatef(cam_yspin, 0.0, 1.0, 0.0);        /* rotate modelmatrix */
    glRotatef(cam_zspin, 0.0, 0.0, 1.0);        /* rotate modelmatrix */

    if (list_inited) glCallList(1);
    else {
	object_define();
	list_inited=TRUE;
    }

    glFlush();

    glXSwapBuffers(dpy,
		   GDK_WINDOW_XWINDOW(GTK_WIDGET(glarea->gtkobject)->window));
    redraw_needed=FALSE;
    return 0;
}

/*
 * Callback for GTK events in glarea 
 *
 */

gint Gtk_GLPort::handle_configure(GdkEventConfigure * event) {

    printf("entry: GDK_CONFIGURE\n");
    redraw_needed=TRUE;
    
    /* Resize OpenGL context in glarea */
    glMatrixMode(GL_PROJECTION);   
    glLoadIdentity();             
    glFrustum(-1.0, 1.0, -1.0, 1.0, 1.5, 20.0);
    glViewport(0, 0, event->width, event->height); 
    glMatrixMode(GL_MODELVIEW);
    printf("exit: GDK_CONFIGURE\n");
    return 0;
}


gint Gtk_GLPort::handle_expose(GdkEventExpose *event)
{
    redraw_needed=TRUE;
    return 1;
}


/**********************************
 *
 * Main function - delegate :)
 *
 **********************************/
int main(int argc, char *argv[])
{
    Gtk_Main myapp(&argc, &argv);
    printf("gui_init\n");
    glp = new Gtk_GLPort(300,300);
    glp->show();

    printf("main widget loop\n");
    myapp.run();

    return 0;
}

