#include "view_window.h"
#include "fractal.h"
#include "defs.h"
#include "statelist.h"

extern StateList fractals;

view_window::view_window(view_window_State * _state) {

    state = _state;

    notebook = new Gtk_Notebook;
    state_options = new Gtk_OptionMenu;
    state_menu = new Gtk_Menu;
    Gtk_MenuItem * item;

    for(GList * ptr = fractals.get_list() ; ptr ; ptr = ptr->next) 
	if(ptr->data) {
	    item = new Gtk_MenuItem(((fractal_State*)ptr->data)->name);
	    state_menu->append(item);
	    item->show();
	}
    
    
    connect_to_method(fractals.append, this, &view_window::append_fractal);

    state_options->set_menu(state_menu);
    //state_menu->show();
    state_options->show();

    Gtk_HBox *option_box = new Gtk_HBox(False);
    Gtk_Label * fract_label = new Gtk_Label("Fractal:");
    option_box->pack_start(fract_label, TRUE, TRUE, 0);
    option_box->pack_start(state_options, TRUE, TRUE, 0);


    fract_label->show();
    option_box->show();
    
    top_box = new Gtk_VBox(False);
    button_box = new Gtk_HBox(False);

    dismiss_button = new Gtk_Button("dismiss");
    revert_button = new Gtk_Button("revert");

    button_box->pack_start(revert_button, TRUE, TRUE, 0);
    button_box->pack_start(dismiss_button, TRUE, TRUE, 0);

    GTK_WIDGET_SET_FLAGS(revert_button->gtkobject, GTK_CAN_DEFAULT);
    GTK_WIDGET_SET_FLAGS(dismiss_button->gtkobject, GTK_CAN_DEFAULT);

    connect_to_signal(dismiss_button->clicked, hide);
    connect_to_method(revert_button->clicked, this, &view_window::revert);
    connect_to_signal(revert_button->clicked, state->view.fract->refresh);

    revert_button->show();
    dismiss_button->show();

    view = state->view.create_gui();
    light = state->light.create_gui();
    model = state->model.create_gui();

    connect_to_signal(state->model.rot.pos->changed, 
		      state->model.fract->refresh);
    gtk_widget_set_events(GTK_WIDGET(state->model.rot.pos->gtkobject),
			  GDK_BUTTON_PRESS_MASK | 
			  GDK_BUTTON_RELEASE_MASK |
			  GDK_EXPOSURE_MASK | 
			  GDK_BUTTON_MOTION_MASK);

    notebook->insert_page(view, new Gtk_Label("Camera"), 0);
    notebook->insert_page(light, new Gtk_Label("Light"), 1);
    notebook->insert_page(model, new Gtk_Label("Fractal"), 2);

    model->show();
    view->show();
    light->show();

    Gtk_HSeparator * sep = new Gtk_HSeparator;

    top_box->pack_start(option_box, 0, TRUE, 2);
    top_box->pack_start(sep, 0, TRUE, 2);
    top_box->pack_start(notebook, 0, TRUE, 0);
    top_box->pack_start(button_box, 0, TRUE, 0);

    sep->show();
    button_box->show();
    notebook->show();
    top_box->show();

    add(top_box);

    show();

}

void view_window::append_fractal(fractal_State *s) {
    Gtk_MenuItem * item = new Gtk_MenuItem(s->name);
    state_menu->append(item);
    item->show();
}

void view_window::revert() {
    state->view.apply();
    state->light.apply();
    state->model.apply();
}
